@@ -1,4 +1,5 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -6,10 +7,14 @@
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE PatternSynonyms #-}
+{-# LANGUAGE ViewPatterns #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Cons
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -24,12 +29,19 @@ module Control.Lens.Cons
   , cons
   , uncons
   , _head, _tail
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern (:>)
+#endif
   -- * Snoc
   , Snoc(..)
   , (|>)
   , snoc
   , unsnoc
   , _init, _last
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern (:<)
+#endif
+
   ) where
 
 import Control.Lens.Equality (simply)
@@ -40,9 +52,11 @@ import Control.Lens.Tuple
 import Control.Lens.Type
 import qualified Data.ByteString      as StrictB
 import qualified Data.ByteString.Lazy as LazyB
+import           Data.List.NonEmpty   (NonEmpty(..))
+import qualified Data.List.NonEmpty   as NonEmpty
 import           Data.Monoid
 import qualified Data.Sequence as Seq
-import           Data.Sequence hiding ((<|), (|>))
+import           Data.Sequence hiding ((<|), (|>), (:<), (:>))
 import qualified Data.Text      as StrictT
 import qualified Data.Text.Lazy as LazyT
 import           Data.Vector (Vector)
@@ -69,6 +83,19 @@ import           Prelude
 infixr 5 <|, `cons`
 infixl 5 |>, `snoc`
 
+#if __GLASGOW_HASKELL__ >= 710
+
+pattern (:<) a s <- (preview _Cons -> Just (a,s)) where
+  (:<) a s = _Cons # (a,s)
+
+infixr 5 :<
+infixl 5 :>
+
+pattern (:>) s a <- (preview _Snoc -> Just (s,a)) where
+  (:>) a s = _Snoc # (a,s)
+
+#endif
+
 ------------------------------------------------------------------------------
 -- Cons
 ------------------------------------------------------------------------------
@@ -94,9 +121,15 @@ instance Cons [a] [b] a b where
     []     -> Left  []
   {-# INLINE _Cons #-}
 
+instance a~b => Cons (NonEmpty a) (NonEmpty b) a b where
+  _Cons = prism' (uncurry NonEmpty.cons) $ \ xyz -> case xyz of
+    (x:|y:z) -> Just (x,y:|z)
+    _        -> Nothing
+  {-# INLINE _Cons #-}
+
 instance Cons (Seq a) (Seq b) a b where
   _Cons = prism (uncurry (Seq.<|)) $ \aas -> case viewl aas of
-    a :< as -> Right (a, as)
+    a Seq.:< as -> Right (a, as)
     EmptyL  -> Left mempty
   {-# INLINE _Cons #-}
 
@@ -319,9 +352,15 @@ instance Snoc [a] [b] a b where
     else Right (Prelude.init aas, Prelude.last aas)
   {-# INLINE _Snoc #-}
 
+instance a~b => Snoc (NonEmpty a) (NonEmpty b) a b where
+  _Snoc = prism' (\(x:|y,z) -> x:|y++[z]) $ \xyz -> case xyz of
+    x:|y
+      | Prelude.null y -> Nothing
+      | otherwise      -> Just (x :| Prelude.init y, Prelude.last y)
+
 instance Snoc (Seq a) (Seq b) a b where
   _Snoc = prism (uncurry (Seq.|>)) $ \aas -> case viewr aas of
-    as :> a -> Right (as, a)
+    as Seq.:> a -> Right (as, a)
     EmptyR  -> Left mempty
   {-# INLINE _Snoc #-}
 
@@ -442,8 +481,8 @@ _init = _Snoc._1
 -- >>> Vector.empty ^? _last
 -- Nothing
 --
--- >>> Vector.fromList "abcde" & _last .~ 'Q'
--- fromList "abcdQ"
+-- >>> (Vector.fromList "abcde" & _last .~ 'Q') == Vector.fromList "abcdQ"
+-- True
 --
 -- @
 -- '_last' :: 'Traversal'' [a] a
