@@ -1,14 +1,20 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE MagicHash #-}
 {-# LANGUAGE DefaultSignatures #-}
+
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE PatternSynonyms #-}
+{-# LANGUAGE ViewPatterns #-}
+#endif
+
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Empty
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -18,9 +24,15 @@
 module Control.Lens.Empty
   (
     AsEmpty(..)
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern Empty
+#endif
   ) where
 
 import Control.Lens.Iso
+#if __GLASGOW_HASKELL__ >= 710
+import Control.Lens.Fold
+#endif
 import Control.Lens.Prism
 import Control.Lens.Review
 import Data.ByteString as StrictB
@@ -57,6 +69,11 @@ class AsEmpty a where
   {-# INLINE _Empty #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 710
+pattern Empty <- (has _Empty -> True) where
+  Empty = review _Empty ()
+#endif
+
 {- Default Monoid instances -}
 instance AsEmpty Ordering
 instance AsEmpty ()
