@@ -21,7 +21,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.TH
--- Copyright   :  (C) 2013-2014 Edward Kmett, 2013 Eric Mertens
+-- Copyright   :  (C) 2013-2015 Edward Kmett and Eric Mertens
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -30,6 +30,7 @@
 ----------------------------------------------------------------------------
 module Control.Lens.Internal.TH where
 
+import Data.Functor.Contravariant
 import Language.Haskell.TH
 import Language.Haskell.TH.Syntax
 import qualified Data.Map as Map
@@ -80,11 +81,11 @@ bndrName :: TyVarBndr -> Name
 bndrName (PlainTV  n  ) = n
 bndrName (KindedTV n _) = n
 
-fromSet :: Ord k => (k -> v) -> Set.Set k -> Map.Map k v
+fromSet :: (k -> v) -> Set.Set k -> Map.Map k v
 #if MIN_VERSION_containers(0,5,0)
 fromSet = Map.fromSet
 #else
-fromSet f x = Map.fromList [ (k,f k) | k <- Set.toList x ]
+fromSet f x = Map.fromDistinctAscList [ (k,f k) | k <- Set.toAscList x ]
 #endif
 
 ------------------------------------------------------------------------
@@ -161,8 +162,12 @@ prismValName             = mkLensName_v "Control.Lens.Prism" "prism"
 untoValName             :: Name
 untoValName              = mkLensName_v "Control.Lens.Review" "unto"
 
-coerceValName           :: Name
-coerceValName            = mkLensName_v "Control.Lens.Internal.Getter" "coerce"
+phantomValName          :: Name
+phantomValName           = mkLensName_v "Control.Lens.Internal.TH" "phantom2"
+
+phantom2 :: (Functor f, Contravariant f) => f a -> f b
+phantom2 = phantom
+{-# INLINE phantom2 #-}
 
 composeValName          :: Name
 composeValName           = mkNameG_v "base" "GHC.Base" "."
