@@ -5,17 +5,22 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-#ifdef TRUSTWORTHY
-{-# LANGUAGE Trustworthy #-}
-#endif
 
 #ifndef MIN_VERSION_mtl
 #define MIN_VERSION_mtl(x,y,z) 1
 #endif
+
+#ifndef MIN_VERSION_profunctors
+#define MIN_VERSION_profunctors(x,y,z) 1
+#endif
+
+#if __GLASGOW_HASKELL__ < 708 || !(MIN_VERSION_profunctors(4,4,0))
+{-# LANGUAGE Trustworthy #-}
+#endif
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -120,6 +125,9 @@ module Control.Lens.Lens
   , Context(..)
   , Context'
   , locus
+
+  -- * Lens fusion
+  , fusing
   ) where
 
 import Control.Applicative
@@ -130,12 +138,17 @@ import Control.Lens.Internal.Getter
 import Control.Lens.Internal.Indexed
 import Control.Lens.Type
 import Control.Monad.State as State
+import Data.Functor.Yoneda
 import Data.Monoid
 import Data.Profunctor
 import Data.Profunctor.Rep
+import Data.Profunctor.Sieve
 import Data.Profunctor.Unsafe
 import Data.Void
 import Prelude
+#if __GLASGOW_HASKELL__ >= 710
+import Data.Function ((&))
+#endif
 
 #ifdef HLINT
 {-# ANN module "HLint: ignore Use ***" #-}
@@ -145,6 +158,7 @@ import Prelude
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Control.Lens
 -- >>> import Control.Monad.State
+-- >>> import Data.Char (chr)
 -- >>> import Debug.SimpleReflect.Expr
 -- >>> import Debug.SimpleReflect.Vars as Vars hiding (f,g,h)
 -- >>> let f :: Expr -> Expr; f = Debug.SimpleReflect.Vars.f
@@ -159,7 +173,7 @@ infixr 4 %%@~, <%@~, <<%@~, %%~, <+~, <*~, <-~, <//~, <^~, <^^~, <**~, <&&~, <||
 infix  4 %%@=, <%@=, <<%@=, %%=, <+=, <*=, <-=, <//=, <^=, <^^=, <**=, <&&=, <||=, <<>=, <%=, <<%=, <<.=, <#=, #=, #%=, <#%=, #%%=
        , <<+=, <<-=, <<*=, <<//=, <<^=, <<^^=, <<**=, <<||=, <<&&=, <<<>=
 infixr 2 <<~
-infixl 1 &, <&>, ??, &~
+infixl 1 <&>, ??, &~
 
 -------------------------------------------------------------------------------
 -- Lenses
@@ -214,7 +228,7 @@ lens sa sbt afb s = sbt s <$> afb (sa s)
 -- | Build an index-preserving 'Lens' from a 'Control.Lens.Getter.Getter' and a
 -- 'Control.Lens.Setter.Setter'.
 iplens :: (s -> a) -> (s -> b -> t) -> IndexPreservingLens s t a b
-iplens sa sbt pafb = cotabulate $ \ws -> sbt (extract ws) <$> corep pafb (sa <$> ws)
+iplens sa sbt pafb = cotabulate $ \ws -> sbt (extract ws) <$> cosieve pafb (sa <$> ws)
 {-# INLINE iplens #-}
 
 -- | Build an 'IndexedLens' from a 'Control.Lens.Getter.Getter' and
@@ -249,6 +263,9 @@ s &~ l = execState l s
 -- targets of the traversals, extracting an applicative summary of its
 -- actions.
 --
+-- >>> [66,97,116,109,97,110] & each %%~ \a -> ("na", chr a)
+-- ("nananananana","Batman")
+--
 -- For all that the definition of this combinator is just:
 --
 -- @
@@ -273,7 +290,7 @@ s &~ l = execState l s
 -- ('%%~') ::             'Lens' s t a b      -> (a -> (r, b)) -> s -> (r, t)
 -- ('%%~') :: 'Monoid' m => 'Control.Lens.Traversal.Traversal' s t a b -> (a -> (m, b)) -> s -> (m, t)
 -- @
-(%%~) :: Optical p q f s t a b -> p a (f b) -> q s (f t)
+(%%~) :: LensLike f s t a b -> (a -> f b) -> s -> f t
 (%%~) = id
 {-# INLINE (%%~) #-}
 
@@ -312,6 +329,8 @@ l %%= f = do
 -- General Purpose Combinators
 -------------------------------------------------------------------------------
 
+
+#if __GLASGOW_HASKELL__ < 710
 -- | Passes the result of the left side to the function on the right side (forward pipe operator).
 --
 -- This is the flipped version of ('$'), which is more common in languages like F# as (@|>@) where it is needed
@@ -336,6 +355,8 @@ l %%= f = do
 (&) :: a -> (a -> b) -> b
 a & f = f a
 {-# INLINE (&) #-}
+infixl 1 &
+#endif
 
 -- | Infix flipped 'fmap'.
 --
@@ -346,8 +367,20 @@ a & f = f a
 as <&> f = f <$> as
 {-# INLINE (<&>) #-}
 
--- | This is convenient to 'flip' argument order of composite functions.
+-- | This is convenient to 'flip' argument order of composite functions defined as:
 --
+-- @
+-- fab ?? a = fmap ($ a) fab
+-- @
+-- 
+-- For the 'Functor' instance @f = ((->) r)@ you can reason about this function as if the definition was @('??') ≡ 'flip'@:
+-- 
+-- >>> (h ?? x) a
+-- h a x
+-- 
+-- >>> execState ?? [] $ modify (1:)
+-- [1]
+-- 
 -- >>> over _2 ?? ("hello","world") $ length
 -- ("hello",5)
 --
@@ -375,8 +408,8 @@ fab ?? a = fmap ($ a) fab
 -- [2,1]
 inside :: Corepresentable p => ALens s t a b -> Lens (p e s) (p e t) (p e a) (p e b)
 inside l f es = o <$> f i where
-  i = cotabulate $ \ e -> ipos $ l sell (corep es e)
-  o ea = cotabulate $ \ e -> ipeek (corep ea e) $ l sell (corep es e)
+  i = cotabulate $ \ e -> ipos $ l sell (cosieve es e)
+  o ea = cotabulate $ \ e -> ipeek (cosieve ea e) $ l sell (cosieve es e)
 {-# INLINE inside #-}
 
 {-
@@ -433,7 +466,7 @@ choosing _ r f (Right a') = Right <$> r f a'
 -- 'chosen' f ('Right' a) = 'Right' '<$>' f a
 -- @
 chosen :: IndexPreservingLens (Either a a) (Either b b) a b
-chosen pafb = cotabulate $ \weaa -> corep (either id id `lmap` pafb) weaa <&> \b -> case extract weaa of
+chosen pafb = cotabulate $ \weaa -> cosieve (either id id `lmap` pafb) weaa <&> \b -> case extract weaa of
   Left _  -> Left  b
   Right _ -> Right b
 {-# INLINE chosen #-}
@@ -498,7 +531,7 @@ cloneLens l afb s = runPretext (l sell s) afb
 -- | Clone a 'Lens' as an 'IndexedPreservingLens' that just passes through whatever
 -- index is on any 'IndexedLens', 'IndexedFold', 'IndexedGetter' or  'IndexedTraversal' it is composed with.
 cloneIndexPreservingLens :: ALens s t a b -> IndexPreservingLens s t a b
-cloneIndexPreservingLens l pafb = cotabulate $ \ws -> runPretext (l sell (extract ws)) $ \a -> corep pafb (a <$ ws)
+cloneIndexPreservingLens l pafb = cotabulate $ \ws -> runPretext (l sell (extract ws)) $ \a -> cosieve pafb (a <$ ws)
 {-# INLINE cloneIndexPreservingLens #-}
 
 -- | Clone an 'IndexedLens' as an 'IndexedLens' with the same index.
@@ -519,8 +552,8 @@ cloneIndexedLens l f s = runPretext (l sell s) (Indexed (indexed f))
 -- ('<%~') ::             'Control.Lens.Iso.Iso' s t a b       -> (a -> b) -> s -> (b, t)
 -- ('<%~') :: 'Monoid' b => 'Control.Lens.Traversal.Traversal' s t a b -> (a -> b) -> s -> (b, t)
 -- @
-(<%~) :: Profunctor p => Optical p q ((,) b) s t a b -> p a b -> q s (b, t)
-l <%~ f = l $ rmap (\t -> (t, t)) f
+(<%~) :: LensLike ((,) b) s t a b -> (a -> b) -> s -> (b, t)
+l <%~ f = l $ (\t -> (t, t)) . f
 {-# INLINE (<%~) #-}
 
 -- | Increment the target of a numerically valued 'Lens' and return the result.
@@ -531,7 +564,7 @@ l <%~ f = l $ rmap (\t -> (t, t)) f
 -- ('<+~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<+~') :: 'Num' a => 'Control.Lens.Iso.Iso'' s a  -> a -> s -> (a, s)
 -- @
-(<+~) :: Num a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<+~) :: Num a => LensLike ((,)a) s t a a -> a -> s -> (a, t)
 l <+~ a = l <%~ (+ a)
 {-# INLINE (<+~) #-}
 
@@ -543,7 +576,7 @@ l <+~ a = l <%~ (+ a)
 -- ('<-~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<-~') :: 'Num' a => 'Control.Lens.Iso.Iso'' s a  -> a -> s -> (a, s)
 -- @
-(<-~) :: Num a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<-~) :: Num a => LensLike ((,)a) s t a a -> a -> s -> (a, t)
 l <-~ a = l <%~ subtract a
 {-# INLINE (<-~) #-}
 
@@ -556,7 +589,7 @@ l <-~ a = l <%~ subtract a
 -- ('<*~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<*~') :: 'Num' a => 'Control.Lens.Iso.Iso''  s a -> a -> s -> (a, s)
 -- @
-(<*~) :: Num a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<*~) :: Num a => LensLike ((,)a) s t a a -> a -> s -> (a, t)
 l <*~ a = l <%~ (* a)
 {-# INLINE (<*~) #-}
 
@@ -568,7 +601,7 @@ l <*~ a = l <%~ (* a)
 -- ('<//~') :: 'Fractional' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<//~') :: 'Fractional' a => 'Control.Lens.Iso.Iso''  s a -> a -> s -> (a, s)
 -- @
-(<//~) :: Fractional a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<//~) :: Fractional a => LensLike ((,)a) s t a a -> a -> s -> (a, t)
 l <//~ a = l <%~ (/ a)
 {-# INLINE (<//~) #-}
 
@@ -581,7 +614,7 @@ l <//~ a = l <%~ (/ a)
 -- ('<^~') :: ('Num' a, 'Integral' e) => 'Lens'' s a -> e -> s -> (a, s)
 -- ('<^~') :: ('Num' a, 'Integral' e) => 'Control.Lens.Iso.Iso'' s a -> e -> s -> (a, s)
 -- @
-(<^~) :: (Num a, Integral e) => Optical (->) q ((,)a) s t a a -> e -> q s (a, t)
+(<^~) :: (Num a, Integral e) => LensLike ((,)a) s t a a -> e -> s -> (a, t)
 l <^~ e = l <%~ (^ e)
 {-# INLINE (<^~) #-}
 
@@ -594,7 +627,7 @@ l <^~ e = l <%~ (^ e)
 -- ('<^^~') :: ('Fractional' a, 'Integral' e) => 'Lens'' s a -> e -> s -> (a, s)
 -- ('<^^~') :: ('Fractional' a, 'Integral' e) => 'Control.Lens.Iso.Iso'' s a -> e -> s -> (a, s)
 -- @
-(<^^~) :: (Fractional a, Integral e) => Optical (->) q ((,)a) s t a a -> e -> q s (a, t)
+(<^^~) :: (Fractional a, Integral e) => LensLike ((,)a) s t a a -> e -> s -> (a, t)
 l <^^~ e = l <%~ (^^ e)
 {-# INLINE (<^^~) #-}
 
@@ -607,7 +640,7 @@ l <^^~ e = l <%~ (^^ e)
 -- ('<**~') :: 'Floating' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<**~') :: 'Floating' a => 'Control.Lens.Iso.Iso'' s a  -> a -> s -> (a, s)
 -- @
-(<**~) :: Floating a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<**~) :: Floating a => LensLike ((,)a) s t a a -> a -> s -> (a, t)
 l <**~ a = l <%~ (** a)
 {-# INLINE (<**~) #-}
 
@@ -619,7 +652,7 @@ l <**~ a = l <%~ (** a)
 -- ('<||~') :: 'Lens'' s 'Bool' -> 'Bool' -> s -> ('Bool', s)
 -- ('<||~') :: 'Control.Lens.Iso.Iso'' s 'Bool'  -> 'Bool' -> s -> ('Bool', s)
 -- @
-(<||~) :: Optical (->) q ((,)Bool) s t Bool Bool -> Bool -> q s (Bool, t)
+(<||~) :: LensLike ((,)Bool) s t Bool Bool -> Bool -> s -> (Bool, t)
 l <||~ b = l <%~ (|| b)
 {-# INLINE (<||~) #-}
 
@@ -631,7 +664,7 @@ l <||~ b = l <%~ (|| b)
 -- ('<&&~') :: 'Lens'' s 'Bool' -> 'Bool' -> s -> ('Bool', s)
 -- ('<&&~') :: 'Control.Lens.Iso.Iso'' s 'Bool'  -> 'Bool' -> s -> ('Bool', s)
 -- @
-(<&&~) :: Optical (->) q ((,)Bool) s t Bool Bool -> Bool -> q s (Bool, t)
+(<&&~) :: LensLike ((,)Bool) s t Bool Bool -> Bool -> s -> (Bool, t)
 l <&&~ b = l <%~ (&& b)
 {-# INLINE (<&&~) #-}
 
@@ -644,7 +677,7 @@ l <&&~ b = l <%~ (&& b)
 -- ('<<%~') ::             'Control.Lens.Iso.Iso' s t a b       -> (a -> b) -> s -> (a, t)
 -- ('<<%~') :: 'Monoid' a => 'Control.Lens.Traversal.Traversal' s t a b -> (a -> b) -> s -> (a, t)
 -- @
-(<<%~) :: Strong p => Optical p q ((,)a) s t a b -> p a b -> q s (a, t)
+(<<%~) :: LensLike ((,)a) s t a b -> (a -> b) -> s -> (a, t)
 (<<%~) l = l . lmap (\a -> (a, a)) . second'
 {-# INLINE (<<%~) #-}
 
@@ -657,7 +690,7 @@ l <&&~ b = l <%~ (&& b)
 -- ('<<.~') ::             'Control.Lens.Iso.Iso' s t a b       -> b -> s -> (a, t)
 -- ('<<.~') :: 'Monoid' a => 'Control.Lens.Traversal.Traversal' s t a b -> b -> s -> (a, t)
 -- @
-(<<.~) :: Optical (->) q ((,)a) s t a b -> b -> q s (a, t)
+(<<.~) :: LensLike ((,)a) s t a b -> b -> s -> (a, t)
 l <<.~ b = l $ \a -> (a, b)
 {-# INLINE (<<.~) #-}
 
@@ -675,7 +708,7 @@ l <<.~ b = l $ \a -> (a, b)
 -- ('<<+~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<<+~') :: 'Num' a => 'Iso'' s a -> a -> s -> (a, s)
 -- @
-(<<+~) :: Num a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+(<<+~) :: Num a => LensLike' ((,) a) s a -> a -> s -> (a, s)
 l <<+~ b = l $ \a -> (a, a + b)
 {-# INLINE (<<+~) #-}
 
@@ -693,7 +726,7 @@ l <<+~ b = l $ \a -> (a, a + b)
 -- ('<<-~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<<-~') :: 'Num' a => 'Iso'' s a -> a -> s -> (a, s)
 -- @
-(<<-~) :: Num a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+(<<-~) :: Num a => LensLike' ((,) a) s a -> a -> s -> (a, s)
 l <<-~ b = l $ \a -> (a, a - b)
 {-# INLINE (<<-~) #-}
 
@@ -711,7 +744,7 @@ l <<-~ b = l $ \a -> (a, a - b)
 -- ('<<*~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<<*~') :: 'Num' a => 'Iso'' s a -> a -> s -> (a, s)
 -- @
-(<<*~) :: Num a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+(<<*~) :: Num a => LensLike' ((,) a) s a -> a -> s -> (a, s)
 l <<*~ b = l $ \a -> (a, a * b)
 {-# INLINE (<<*~) #-}
 
@@ -729,7 +762,7 @@ l <<*~ b = l $ \a -> (a, a * b)
 -- ('<<//~') :: Fractional a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<<//~') :: Fractional a => 'Iso'' s a -> a -> s -> (a, s)
 -- @
-(<<//~) :: Fractional a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+(<<//~) :: Fractional a => LensLike' ((,) a) s a -> a -> s -> (a, s)
 l <<//~ b = l $ \a -> (a, a / b)
 {-# INLINE (<<//~) #-}
 
@@ -741,7 +774,7 @@ l <<//~ b = l $ \a -> (a, a / b)
 -- ('<<^~') :: ('Num' a, 'Integral' e) => 'Lens'' s a -> e -> s -> (a, s)
 -- ('<<^~') :: ('Num' a, 'Integral' e) => 'Iso'' s a -> e -> s -> (a, s)
 -- @
-(<<^~) :: (Num a, Integral e) => Optical' (->) q ((,) a) s a -> e -> q s (a, s)
+(<<^~) :: (Num a, Integral e) => LensLike' ((,) a) s a -> e -> s -> (a, s)
 l <<^~ e = l $ \a -> (a, a ^ e)
 {-# INLINE (<<^~) #-}
 
@@ -753,7 +786,7 @@ l <<^~ e = l $ \a -> (a, a ^ e)
 -- ('<<^^~') :: ('Fractional' a, 'Integral' e) => 'Lens'' s a -> e -> s -> (a, s)
 -- ('<<^^~') :: ('Fractional' a, 'Integral' e) => 'Iso'' s a -> e -> S -> (a, s)
 -- @
-(<<^^~) :: (Fractional a, Integral e) => Optical' (->) q ((,) a) s a -> e -> q s (a, s)
+(<<^^~) :: (Fractional a, Integral e) => LensLike' ((,) a) s a -> e -> s -> (a, s)
 l <<^^~ e = l $ \a -> (a, a ^^ e)
 {-# INLINE (<<^^~) #-}
 
@@ -771,7 +804,7 @@ l <<^^~ e = l $ \a -> (a, a ^^ e)
 -- ('<<**~') :: 'Floating' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<<**~') :: 'Floating' a => 'Iso'' s a -> a -> s -> (a, s)
 -- @
-(<<**~) :: Floating a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+(<<**~) :: Floating a => LensLike' ((,) a) s a -> a -> s -> (a, s)
 l <<**~ e = l $ \a -> (a, a ** e)
 {-# INLINE (<<**~) #-}
 
@@ -789,7 +822,7 @@ l <<**~ e = l $ \a -> (a, a ** e)
 -- ('<<||~') :: 'Lens'' s 'Bool' -> 'Bool' -> s -> ('Bool', s)
 -- ('<<||~') :: 'Iso'' s 'Bool' -> 'Bool' -> s -> ('Bool', s)
 -- @
-(<<||~) :: Optical' (->) q ((,) Bool) s Bool -> Bool -> q s (Bool, s)
+(<<||~) :: LensLike' ((,) Bool) s Bool -> Bool -> s -> (Bool, s)
 l <<||~ b = l $ \a -> (a, b || a)
 {-# INLINE (<<||~) #-}
 
@@ -807,7 +840,7 @@ l <<||~ b = l $ \a -> (a, b || a)
 -- ('<<&&~') :: 'Lens'' s Bool -> Bool -> s -> (Bool, s)
 -- ('<<&&~') :: 'Iso'' s Bool -> Bool -> s -> (Bool, s)
 -- @
-(<<&&~) :: Optical' (->) q ((,) Bool) s Bool -> Bool -> q s (Bool, s)
+(<<&&~) :: LensLike' ((,) Bool) s Bool -> Bool -> s -> (Bool, s)
 l <<&&~ b = l $ \a -> (a, b && a)
 {-# INLINE (<<&&~) #-}
 
@@ -825,7 +858,7 @@ l <<&&~ b = l $ \a -> (a, b && a)
 -- ('<<<>~') :: 'Monoid' r => 'Lens'' s r -> r -> s -> (r, s)
 -- ('<<<>~') :: 'Monoid' r => 'Iso'' s r -> r -> s -> (r, s)
 -- @
-(<<<>~) :: Monoid r => Optical' (->) q ((,) r) s r -> r -> q s (r, s)
+(<<<>~) :: Monoid r => LensLike' ((,) r) s r -> r -> s -> (r, s)
 l <<<>~ b = l $ \a -> (a, a `mappend` b)
 {-# INLINE (<<<>~) #-}
 
@@ -846,8 +879,8 @@ l <<<>~ b = l $ \a -> (a, a `mappend` b)
 -- ('<%=') :: 'MonadState' s m             => 'Control.Lens.Iso.Iso'' s a       -> (a -> a) -> m a
 -- ('<%=') :: ('MonadState' s m, 'Monoid' a) => 'Control.Lens.Traversal.Traversal'' s a -> (a -> a) -> m a
 -- @
-(<%=) :: (Profunctor p, MonadState s m) => Over p ((,)b) s s a b -> p a b -> m b
-l <%= f = l %%= rmap (\b -> (b, b)) f
+(<%=) :: MonadState s m => LensLike ((,)b) s s a b -> (a -> b) -> m b
+l <%= f = l %%= (\b -> (b, b)) . f
 {-# INLINE (<%=) #-}
 
 
@@ -982,7 +1015,7 @@ l <&&= b = l <%= (&& b)
 -- @
 -- ('<<%=') :: 'MonadState' s m             => 'Lens'' s a      -> (a -> a) -> m a
 -- ('<<%=') :: 'MonadState' s m             => 'Control.Lens.Iso.Iso'' s a       -> (a -> a) -> m a
--- ('<<%=') :: ('MonadState' s m, 'Monoid' b) => 'Control.Lens.Traversal.Traversal'' s a -> (a -> a) -> m a
+-- ('<<%=') :: ('MonadState' s m, 'Monoid' a) => 'Control.Lens.Traversal.Traversal'' s a -> (a -> a) -> m a
 -- @
 --
 -- @('<<%=') :: 'MonadState' s m => 'LensLike' ((,)a) s s a b -> (a -> b) -> m a@
@@ -1157,7 +1190,7 @@ l <<~ mb = do
 -- return the result.
 --
 -- When you do not need the result of the operation, ('Control.Lens.Setter.<>~') is more flexible.
-(<<>~) :: Monoid m => Optical (->) q ((,)m) s t m m -> m -> q s (m, t)
+(<<>~) :: Monoid m => LensLike ((,)m) s t m m -> m -> s -> (m, t)
 l <<>~ m = l <%~ (`mappend` m)
 {-# INLINE (<<>~) #-}
 
@@ -1195,7 +1228,9 @@ overA l p = arr (\s -> let (Context f a) = l sell s in (f, a))
 -- adjust all of the targets of an 'Control.Lens.Traversal.IndexedTraversal' and return a monoidal summary
 -- along with the answer.
 --
--- @l '<%~' f ≡ l '<%@~' 'const' f@
+-- @
+-- l '<%~' f ≡ l '<%@~' 'const' f
+-- @
 --
 -- When you do not need access to the index then ('<%~') is more liberal in what it can accept.
 --
@@ -1205,7 +1240,7 @@ overA l p = arr (\s -> let (Context f a) = l sell s in (f, a))
 -- ('<%@~') ::             'IndexedLens' i s t a b      -> (i -> a -> b) -> s -> (b, t)
 -- ('<%@~') :: 'Monoid' b => 'Control.Lens.Traversal.IndexedTraversal' i s t a b -> (i -> a -> b) -> s -> (b, t)
 -- @
-(<%@~) :: Optical (Indexed i) q ((,) b) s t a b -> (i -> a -> b) -> q s (b, t)
+(<%@~) :: Over (Indexed i) ((,) b) s t a b -> (i -> a -> b) -> s -> (b, t)
 l <%@~ f = l (Indexed $ \i a -> let b = f i a in (b, b))
 {-# INLINE (<%@~) #-}
 
@@ -1217,7 +1252,7 @@ l <%@~ f = l (Indexed $ \i a -> let b = f i a in (b, b))
 -- ('<<%@~') ::             'IndexedLens' i s t a b      -> (i -> a -> b) -> s -> (a, t)
 -- ('<<%@~') :: 'Monoid' a => 'Control.Lens.Traversal.IndexedTraversal' i s t a b -> (i -> a -> b) -> s -> (a, t)
 -- @
-(<<%@~) :: Optical (Indexed i) q ((,) a) s t a b -> (i -> a -> b) -> q s (a, t)
+(<<%@~) :: Over (Indexed i) ((,) a) s t a b -> (i -> a -> b) -> s -> (a, t)
 l <<%@~ f = l $ Indexed $ \i a -> second' (f i) (a,a)
 
 {-# INLINE (<<%@~) #-}
@@ -1226,7 +1261,9 @@ l <<%@~ f = l $ Indexed $ \i a -> second' (f i) (a,a)
 -- adjust all of the targets of an 'Control.Lens.Traversal.IndexedTraversal' and return a monoidal summary
 -- of the supplementary results and the answer.
 --
--- @('%%@~') ≡ 'Control.Lens.Indexed.withIndex'@
+-- @
+-- ('%%@~') ≡ 'Control.Lens.Indexed.withIndex'
+-- @
 --
 -- @
 -- ('%%@~') :: 'Functor' f => 'IndexedLens' i s t a b      -> (i -> a -> f b) -> s -> f t
@@ -1248,7 +1285,9 @@ l <<%@~ f = l $ Indexed $ \i a -> second' (f i) (a,a)
 -- adjust all of the targets of an 'Control.Lens.Traversal.IndexedTraversal' within the current state, and
 -- return a monoidal summary of the supplementary results.
 --
--- @l '%%@=' f ≡ 'state' (l '%%@~' f)@
+-- @
+-- l '%%@=' f ≡ 'state' (l '%%@~' f)
+-- @
 --
 -- @
 -- ('%%@=') :: 'MonadState' s m                 => 'IndexedLens' i s s a b      -> (i -> a -> (r, b)) -> s -> m r
@@ -1416,3 +1455,26 @@ devoid _ = absurd
 united :: Lens' a ()
 united f v = f () <&> \ () -> v
 {-# INLINE united #-}
+
+-- | Fuse a composition of lenses using 'Yoneda' to provide 'fmap' fusion.
+--
+-- In general, given a pair of lenses 'foo' and 'bar'
+--
+-- @
+-- fusing (foo.bar) = foo.bar
+-- @
+--
+-- however, @foo@ and @bar@ are either going to 'fmap' internally or they are trivial.
+--
+-- 'fusing' exploits the 'Yoneda' lemma to merge these separate uses into a single 'fmap'.
+--
+-- This is particularly effective when the choice of functor 'f' is unknown at compile
+-- time or when the 'Lens' @foo.bar@ in the above description is recursive or complex
+-- enough to prevent inlining.
+--
+-- @
+-- 'fusing' :: 'Lens' s t a b -> 'Lens' s t a b
+-- @
+fusing :: Functor f => LensLike (Yoneda f) s t a b -> LensLike f s t a b
+fusing t = \f -> lowerYoneda . t (liftYoneda . f)
+{-# INLINE fusing #-}
