@@ -4,10 +4,15 @@
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
+
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
+
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Prism
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -53,12 +58,15 @@ import Control.Monad
 import Data.Functor.Identity
 import Data.Profunctor
 import Data.Profunctor.Rep
+import Data.Profunctor.Sieve
 import Data.Traversable
 import Data.Void
-#ifndef SAFE
-import Unsafe.Coerce
-#else
+#if MIN_VERSION_base(4,7,0)
+import Data.Coerce
+#elif defined(SAFE)
 import Data.Profunctor.Unsafe
+#else
+import Unsafe.Coerce
 #endif
 import Prelude
 
@@ -89,7 +97,10 @@ type APrism' s a = APrism s s a a
 
 -- | Convert 'APrism' to the pair of functions that characterize it.
 withPrism :: APrism s t a b -> ((b -> t) -> (s -> Either t a) -> r) -> r
-#ifdef SAFE
+#if MIN_VERSION_base(4,7,0)
+withPrism k f = case coerce (k (Market Identity Right)) of
+  Market bt seta -> f bt seta
+#elif defined(SAFE)
 withPrism k f = case k (Market Identity Right) of
   Market bt seta -> f (runIdentity #. bt) (either (Left . runIdentity) Right . seta)
 #else
@@ -130,7 +141,7 @@ prism' bs sma = prism bs (\s -> maybe (Left s) Right (sma s))
 -- TODO: can we make this work with merely Strong?
 outside :: Representable p => APrism s t a b -> Lens (p t r) (p s r) (p b r) (p a r)
 outside k = withPrism k $ \bt seta f ft ->
-  f (lmap bt ft) <&> \fa -> tabulate $ either (rep ft) (rep fa) . seta
+  f (lmap bt ft) <&> \fa -> tabulate $ either (sieve ft) (sieve fa) . seta
 {-# INLINE outside #-}
 
 -- | Given a pair of prisms, project sums.
@@ -160,6 +171,12 @@ aside k =
 {-# INLINE aside #-}
 
 -- | 'lift' a 'Prism' through a 'Traversable' functor, giving a Prism that matches only if all the elements of the container match the 'Prism'.
+--
+-- >>> [Left 1, Right "foo", Left 4, Right "woot"]^..below _Right
+-- []
+--
+-- >>> [Right "hail hydra!", Right "foo", Right "blah", Right "woot"]^..below _Right
+-- [["hail hydra!","foo","blah","woot"]]
 below :: Traversable f => APrism' s a -> Prism' (f s) (f a)
 below k =
   withPrism k     $ \bt seta ->
@@ -316,14 +333,24 @@ only a = prism' (\() -> a) $ guard . (a ==)
 {-# INLINE only #-}
 
 
--- | This 'Prism' compares for approximate equality with a given value and a predicate for testing.
+-- | This 'Prism' compares for approximate equality with a given value and a predicate for testing, 
+-- an example where the value is the empty list and the predicate checks that a list is empty (same
+-- as 'Control.Lens.Empty._Empty' with the 'Control.Lens.Empty.AsEmpty' list instance):
+-- 
+-- >>> nearly [] null # ()
+-- []
+-- >>> [1,2,3,4] ^? nearly [] null
+-- Nothing
+-- 
+-- @'nearly' [] 'Prelude.null' :: 'Prism'' [a] ()@
 --
 -- To comply with the 'Prism' laws the arguments you supply to @nearly a p@ are somewhat constrained.
 --
 -- We assume @p x@ holds iff @x ≡ a@. Under that assumption then this is a valid 'Prism'.
 --
 -- This is useful when working with a type where you can test equality for only a subset of its
--- values, and the prism selects such a value.
+-- values, and the prism selects such a value. 
+
 nearly :: a -> (a -> Bool) -> Prism' a ()
 nearly a p = prism' (\() -> a) $ guard . p
 {-# INLINE nearly #-}
