@@ -5,7 +5,7 @@
 ------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Reified
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -18,23 +18,24 @@ import Control.Applicative
 import Control.Arrow
 import qualified Control.Category as Cat
 import Control.Comonad
-import Control.Lens.Action
 import Control.Lens.Fold
 import Control.Lens.Getter
 import Control.Lens.Internal.Indexed
-import Control.Lens.Traversal (ignored,beside)
+import Control.Lens.Traversal (ignored)
 import Control.Lens.Type
 import Control.Monad
 import Control.Monad.Reader.Class
 import Data.Distributive
 import Data.Foldable
 import Data.Functor.Compose
+import Data.Functor.Contravariant
 import Data.Functor.Bind
 import Data.Functor.Extend
 import Data.Functor.Identity
 import Data.Functor.Plus
 import Data.Profunctor
 import Data.Profunctor.Rep
+import Data.Profunctor.Sieve
 import Data.Semigroup
 
 -- $setup
@@ -168,24 +169,31 @@ instance Profunctor ReifiedGetter where
   rmap f l    = Getter $ runGetter l.to f
   {-# INLINE rmap #-}
 
+instance Cosieve ReifiedGetter Identity where
+  cosieve (Getter l) = view l . runIdentity
+
 instance Corepresentable ReifiedGetter where
   type Corep ReifiedGetter = Identity
   cotabulate f = Getter $ to (f . Identity)
-  corep (Getter l) = view l . runIdentity
+
+instance Sieve ReifiedGetter Identity where
+  sieve (Getter l) = Identity . view l
 
 instance Representable ReifiedGetter where
   type Rep ReifiedGetter = Identity
   tabulate f = Getter $ to (runIdentity . f)
-  rep (Getter l) = Identity . view l
+
+instance Costrong ReifiedGetter where
+  unfirst l = Getter $ to $ unfirst $ view (runGetter l)
 
 instance Conjoined ReifiedGetter
 
 instance Strong ReifiedGetter where
   first' l = Getter $ \f (s,c) ->
-    coerce $ runGetter l (dimap (flip (,) c) coerce f) s
+    phantom $ runGetter l (dimap (flip (,) c) phantom f) s
   {-# INLINE first' #-}
   second' l = Getter $ \f (c,s) ->
-    coerce $ runGetter l (dimap ((,) c) coerce f) s
+    phantom $ runGetter l (dimap ((,) c) phantom f) s
   {-# INLINE second' #-}
 
 instance Choice ReifiedGetter where
@@ -240,19 +248,21 @@ instance Profunctor (ReifiedIndexedGetter i) where
   dimap f g l = IndexedGetter (to f . runIndexedGetter l . to g)
   {-# INLINE dimap #-}
 
+instance Sieve (ReifiedIndexedGetter i) ((,) i) where
+  sieve = iview . runIndexedGetter
+  {-# INLINE sieve #-}
+
 instance Representable (ReifiedIndexedGetter i) where
   type Rep (ReifiedIndexedGetter i) = (,) i
   tabulate f = IndexedGetter $ ito f
   {-# INLINE tabulate #-}
-  rep = iview . runIndexedGetter
-  {-# INLINE rep #-}
 
 instance Strong (ReifiedIndexedGetter i) where
   first' l = IndexedGetter $ \f (s,c) ->
-    coerce $ runIndexedGetter l (dimap (flip (,) c) coerce f) s
+    phantom $ runIndexedGetter l (dimap (flip (,) c) phantom f) s
   {-# INLINE first' #-}
   second' l = IndexedGetter $ \f (c,s) ->
-    coerce $ runIndexedGetter l (dimap ((,) c) coerce f) s
+    phantom $ runIndexedGetter l (dimap ((,) c) phantom f) s
   {-# INLINE second' #-}
 
 instance Functor (ReifiedIndexedGetter i s) where
@@ -263,7 +273,7 @@ instance Semigroup i => Apply (ReifiedIndexedGetter i s) where
   IndexedGetter mf <.> IndexedGetter ma = IndexedGetter $ \k s ->
     case iview mf s of
       (i, f) -> case iview ma s of
-        (j, a) -> coerce $ indexed k (i <> j) (f a)
+        (j, a) -> phantom $ indexed k (i <> j) (f a)
   {-# INLINE (<.>) #-}
 
 ------------------------------------------------------------------------------
@@ -288,17 +298,19 @@ instance Profunctor ReifiedFold where
   lmap f l = Fold (to f . runFold l)
   {-# INLINE lmap #-}
 
+instance Sieve ReifiedFold [] where
+  sieve = toListOf . runFold
+
 instance Representable ReifiedFold where
   type Rep ReifiedFold = []
   tabulate f = Fold (folding f)
-  rep = toListOf . runFold
 
 instance Strong ReifiedFold where
   first' l = Fold $ \f (s,c) ->
-    coerce $ runFold l (dimap (flip (,) c) coerce f) s
+    phantom $ runFold l (dimap (flip (,) c) phantom f) s
   {-# INLINE first' #-}
   second' l = Fold $ \f (c,s) ->
-    coerce $ runFold l (dimap ((,) c) coerce f) s
+    phantom $ runFold l (dimap ((,) c) phantom f) s
   {-# INLINE second' #-}
 
 instance Choice ReifiedFold where
@@ -443,137 +455,23 @@ instance Profunctor (ReifiedIndexedFold i) where
   rmap g l = IndexedFold (runIndexedFold l . to g)
   {-# INLINE rmap #-}
 
+instance Sieve (ReifiedIndexedFold i) (Compose [] ((,) i)) where
+  sieve (IndexedFold l) = Compose . itoListOf l
+  {-# INLINE sieve #-}
+
 instance Representable (ReifiedIndexedFold i) where
   type Rep (ReifiedIndexedFold i) = Compose [] ((,) i)
-  tabulate k = IndexedFold $ \f -> coerce . traverse_ (coerce . uncurry (indexed f)) . getCompose . k
+  tabulate k = IndexedFold $ \f -> phantom . traverse_ (phantom . uncurry (indexed f)) . getCompose . k
   {-# INLINE tabulate #-}
-  rep (IndexedFold l) = Compose . itoListOf l
-  {-# INLINE rep #-}
 
 instance Strong (ReifiedIndexedFold i) where
   first' l  = IndexedFold $ \f (s,c) ->
-    coerce $ runIndexedFold l (dimap (flip (,) c) coerce f) s
+    phantom $ runIndexedFold l (dimap (flip (,) c) phantom f) s
   {-# INLINE first' #-}
   second' l = IndexedFold $ \f (c,s) ->
-    coerce $ runIndexedFold l (dimap ((,) c) coerce f) s
+    phantom $ runIndexedFold l (dimap ((,) c) phantom f) s
   {-# INLINE second' #-}
 
-------------------------------------------------------------------------------
--- MonadicFold
-------------------------------------------------------------------------------
-
--- | Reify a 'MonadicFold' so it can be stored safely in a container.
---
-newtype ReifiedMonadicFold m s a = MonadicFold { runMonadicFold :: MonadicFold m s a }
-
-instance Profunctor (ReifiedMonadicFold m) where
-  dimap f g l = MonadicFold (to f . runMonadicFold l . to g)
-  {-# INLINE dimap #-}
-  rmap g l = MonadicFold (runMonadicFold l . to g)
-  {-# INLINE rmap #-}
-  lmap f l = MonadicFold (to f . runMonadicFold l)
-  {-# INLINE lmap #-}
-
-instance Strong (ReifiedMonadicFold m) where
-  first' l = MonadicFold $ \f (s,c) ->
-    coerce $ runMonadicFold l (dimap (flip (,) c) coerce f) s
-  {-# INLINE first' #-}
-  second' l = MonadicFold $ \f (c,s) ->
-    coerce $ runMonadicFold l (dimap ((,) c) coerce f) s
-  {-# INLINE second' #-}
-
-instance Choice (ReifiedMonadicFold m) where
-  left' (MonadicFold l) = MonadicFold $
-    to tuplify.beside (folded.l.to Left) (folded.to Right)
-    where
-      tuplify (Left lval) = (Just lval,Nothing)
-      tuplify (Right rval) = (Nothing,Just rval)
-  {-# INLINE left' #-}
-
-instance Cat.Category (ReifiedMonadicFold m) where
-  id = MonadicFold id
-  l . r = MonadicFold (runMonadicFold r . runMonadicFold l)
-  {-# INLINE (.) #-}
-
-instance Arrow (ReifiedMonadicFold m) where
-  arr f = MonadicFold (to f)
-  {-# INLINE arr #-}
-  first = first'
-  {-# INLINE first #-}
-  second = second'
-  {-# INLINE second #-}
-
-instance ArrowChoice (ReifiedMonadicFold m) where
-  left = left'
-  {-# INLINE left #-}
-  right = right'
-  {-# INLINE right #-}
-
-instance ArrowApply (ReifiedMonadicFold m) where
-  app = MonadicFold $ \cHandler (argFold,b) ->
-     runMonadicFold (pure b >>> argFold) cHandler (argFold,b)
-  {-# INLINE app #-}
-
-instance Functor (ReifiedMonadicFold m s) where
-  fmap f l = MonadicFold (runMonadicFold l.to f)
-  {-# INLINE fmap #-}
-
-instance Apply (ReifiedMonadicFold m s) where
-  mf <.> ma = mf &&& ma >>> (MonadicFold $ to (uncurry ($)))
-  {-# INLINE (<.>) #-}
-
-instance Applicative (ReifiedMonadicFold m s) where
-  pure a = MonadicFold $ folding $ \_ -> [a]
-  {-# INLINE pure #-}
-  mf <*> ma = mf <.> ma
-  {-# INLINE (<*>) #-}
-
-instance Alternative (ReifiedMonadicFold m s) where
-  empty = MonadicFold ignored
-  {-# INLINE empty #-}
-  MonadicFold ma <|> MonadicFold mb = MonadicFold $ to (\x->(x,x)).beside ma mb
-  {-# INLINE (<|>) #-}
-
-instance Bind (ReifiedMonadicFold m s) where
-  ma >>- f = ((ma >>^ f) &&& returnA) >>> app 
-  {-# INLINE (>>-) #-}
-
-instance Monad (ReifiedMonadicFold m s) where
-  return a = MonadicFold $ folding $ \_ -> [a]
-  {-# INLINE return #-}
-  ma >>= f = ((ma >>^ f) &&& returnA) >>> app 
-  {-# INLINE (>>=) #-}
-
-instance MonadReader s (ReifiedMonadicFold m s) where
-  ask = returnA
-  {-# INLINE ask #-}
-  local f ma = f ^>> ma 
-  {-# INLINE local #-}
-
-instance MonadPlus (ReifiedMonadicFold m s) where
-  mzero = empty
-  {-# INLINE mzero #-}
-  mplus = (<|>)
-  {-# INLINE mplus #-}
-
-instance Semigroup (ReifiedMonadicFold m s a) where
-  (<>) = (<|>)
-  {-# INLINE (<>) #-}
-
-instance Monoid (ReifiedMonadicFold m s a) where
-  mempty = MonadicFold ignored
-  {-# INLINE mempty #-}
-  mappend = (<|>)
-  {-# INLINE mappend #-}
-
-instance Alt (ReifiedMonadicFold m s) where
-  (<!>) = (<|>)
-  {-# INLINE (<!>) #-}
-
-instance Plus (ReifiedMonadicFold m s) where
-  zero = MonadicFold ignored
-  {-# INLINE zero #-}
-
 ------------------------------------------------------------------------------
 -- Setter
 ------------------------------------------------------------------------------
