@@ -3,13 +3,18 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE FlexibleContexts #-}
-#ifdef TRUSTWORTHY
+
+#ifndef MIN_VERSION_profunctors
+#define MIN_VERSION_profunctors(x,y,z) 1
+#endif
+
+#if __GLASGOW_HASKELL__ < 708 || !(MIN_VERSION_profunctors(4,4,0))
 {-# LANGUAGE Trustworthy #-}
 #endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Setter
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -52,7 +57,7 @@ module Control.Lens.Setter
   , (.~), (%~)
   , (+~), (-~), (*~), (//~), (^~), (^^~), (**~), (||~), (<>~), (&&~), (<.~), (?~), (<?~)
   -- * State Combinators
-  , assign
+  , assign, modifying
   , (.=), (%=)
   , (+=), (-=), (*=), (//=), (^=), (^^=), (**=), (||=), (<>=), (&&=), (<.=), (?=), (<?=)
   , (<~)
@@ -63,9 +68,9 @@ module Control.Lens.Setter
   -- * Simplified State Setting
   , set'
   -- * Indexed Setters
-  , imapOf, iover
+  , imapOf, iover, iset, imodifying
   , isets
-  , (%@~), (%@=)
+  , (%@~), (.@~), (%@=), (.@=)
   -- * Arrow operators
   , assignA
   -- * Exported for legible error messages
@@ -89,6 +94,7 @@ import Data.Functor.Identity
 import Data.Monoid
 import Data.Profunctor
 import Data.Profunctor.Rep
+import Data.Profunctor.Sieve
 import Data.Profunctor.Unsafe
 import Prelude
 
@@ -110,8 +116,8 @@ import Prelude
 -- >>> let setter :: Expr -> Expr -> Expr; setter = fun "setter"
 -- >>> :set -XNoOverloadedStrings
 
-infixr 4 %@~, .~, +~, *~, -~, //~, ^~, ^^~, **~, &&~, <>~, ||~, %~, <.~, ?~, <?~
-infix  4 %@=, .=, +=, *=, -=, //=, ^=, ^^=, **=, &&=, <>=, ||=, %=, <.=, ?=, <?=
+infixr 4 %@~, .@~, .~, +~, *~, -~, //~, ^~, ^^~, **~, &&~, <>~, ||~, %~, <.~, ?~, <?~
+infix  4 %@=, .@=, .=, +=, *=, -=, //=, ^=, ^^=, **=, &&=, <>=, ||=, %=, <.=, ?=, <?=
 infixr 2 <~
 
 ------------------------------------------------------------------------------
@@ -278,7 +284,7 @@ argument = sets lmap
 -- 'setting' :: ((a -> b) -> s -> t) -> 'Setter' s t a b
 -- @
 setting :: ((a -> b) -> s -> t) -> IndexPreservingSetter s t a b
-setting l pafb = cotabulate $ \ws -> pure $ l (\a -> untainted (corep pafb (a <$ ws))) (extract ws)
+setting l pafb = cotabulate $ \ws -> pure $ l (\a -> untainted (cosieve pafb (a <$ ws))) (extract ws)
 {-# INLINE setting #-}
 
 -- | Build a 'Setter', 'IndexedSetter' or 'IndexPreservingSetter' depending on your choice of 'Profunctor'.
@@ -298,7 +304,7 @@ cloneSetter l afb = taintedDot $ runIdentity #. l (Identity #. untaintedDot afb)
 -- | Build an 'IndexPreservingSetter' from any 'Setter'.
 cloneIndexPreservingSetter :: ASetter s t a b -> IndexPreservingSetter s t a b
 cloneIndexPreservingSetter l pafb = cotabulate $ \ws ->
-    taintedDot runIdentity $ l (\a -> Identity (untainted (corep pafb (a <$ ws)))) (extract ws)
+    taintedDot runIdentity $ l (\a -> Identity (untainted (cosieve pafb (a <$ ws)))) (extract ws)
 {-# INLINE cloneIndexPreservingSetter #-}
 
 -- | Clone an 'IndexedSetter'.
@@ -350,7 +356,7 @@ cloneIndexedSetter l pafb = taintedDot (runIdentity #. l (Indexed $ \i -> Identi
 -- 'over' :: 'Setter' s t a b -> (a -> b) -> s -> t
 -- 'over' :: 'ASetter' s t a b -> (a -> b) -> s -> t
 -- @
-over :: Profunctor p => Setting p s t a b -> p a b -> s -> t
+over :: ASetter s t a b -> (a -> b) -> s -> t
 over l f = runIdentity #. l (Identity #. f)
 {-# INLINE over #-}
 
@@ -441,7 +447,7 @@ set' l b = runIdentity #. l (\_ -> Identity b)
 -- ('%~') :: 'Lens' s t a b      -> (a -> b) -> s -> t
 -- ('%~') :: 'Traversal' s t a b -> (a -> b) -> s -> t
 -- @
-(%~) :: Profunctor p => Setting p s t a b -> p a b -> s -> t
+(%~) :: ASetter s t a b -> (a -> b) -> s -> t
 (%~) = over
 {-# INLINE (%~) #-}
 
@@ -497,7 +503,7 @@ l ?~ b = set l (Just b)
 
 -- | Set with pass-through.
 --
--- This is mostly present for consistency, but may be useful for for chaining assignments.
+-- This is mostly present for consistency, but may be useful for chaining assignments.
 --
 -- If you do not need a copy of the intermediate result, then using @l '.~' t@ directly is a good idea.
 --
@@ -788,10 +794,15 @@ l .= b = State.modify (l .~ b)
 -- @
 -- ('%=') :: 'MonadState' s m => 'ASetter' s s a b -> (a -> b) -> m ()
 -- @
-(%=) :: (Profunctor p, MonadState s m) => Setting p s s a b -> p a b -> m ()
+(%=) :: MonadState s m => ASetter s s a b -> (a -> b) -> m ()
 l %= f = State.modify (l %~ f)
 {-# INLINE (%=) #-}
 
+-- | This is an alias for ('%=').
+modifying :: MonadState s m => ASetter s s a b -> (a -> b) -> m ()
+modifying l f = State.modify (over l f)
+{-# INLINE modifying #-}
+
 -- | Replace the target of a 'Lens' or all of the targets of a 'Setter' or 'Traversal' in our monadic
 -- state with 'Just' a new value, irrespective of the old.
 --
@@ -1123,9 +1134,26 @@ icensoring l uv = censor (iover l uv)
 -- 'iover' :: 'IndexedTraversal' i s t a b -> (i -> a -> b) -> s -> t
 -- @
 iover :: AnIndexedSetter i s t a b -> (i -> a -> b) -> s -> t
-iover l = over l .# Indexed
+iover l f = runIdentity #. l (Identity #. Indexed f)
 {-# INLINE iover #-}
 
+-- | Set with index. Equivalent to 'iover' with the current value ignored.
+--
+-- When you do not need access to the index, then 'set' is more liberal in what it can accept.
+--
+-- @
+-- 'set' l ≡ 'iset' l '.' 'const'
+-- @
+--
+-- @
+-- 'iset' :: 'IndexedSetter' i s t a b    -> (i -> b) -> s -> t
+-- 'iset' :: 'IndexedLens' i s t a b      -> (i -> b) -> s -> t
+-- 'iset' :: 'IndexedTraversal' i s t a b -> (i -> b) -> s -> t
+-- @
+iset :: AnIndexedSetter i s t a b -> (i -> b) -> s -> t
+iset l = iover l . (const .)
+{-# INLINE iset #-}
+
 -- | Build an 'IndexedSetter' from an 'Control.Lens.Indexed.imap'-like function.
 --
 -- Your supplied function @f@ is required to satisfy:
@@ -1142,8 +1170,8 @@ iover l = over l .# Indexed
 -- 'iover' '.' 'isets' ≡ 'id'
 -- @
 --
--- Another way to view 'sets' is that it takes a \"semantic editor combinator\"
--- and transforms it into a 'Setter'.
+-- Another way to view 'isets' is that it takes a \"semantic editor combinator\"
+-- which has been modified to carry an index and transforms it into a 'IndexedSetter'.
 isets :: ((i -> a -> b) -> s -> t) -> IndexedSetter i s t a b
 isets f = sets (f . indexed)
 {-# INLINE isets #-}
@@ -1152,10 +1180,10 @@ isets f = sets (f . indexed)
 -- with access to the index.
 --
 -- @
--- ('%@~') ≡ 'imapOf'
+-- ('%@~') ≡ 'iover'
 -- @
 --
--- When you do not need access to the index then ('%@~') is more liberal in what it can accept.
+-- When you do not need access to the index then ('%~') is more liberal in what it can accept.
 --
 -- @
 -- l '%~' f ≡ l '%@~' 'const' f
@@ -1167,9 +1195,31 @@ isets f = sets (f . indexed)
 -- ('%@~') :: 'IndexedTraversal' i s t a b -> (i -> a -> b) -> s -> t
 -- @
 (%@~) :: AnIndexedSetter i s t a b -> (i -> a -> b) -> s -> t
-l %@~ f = l %~ Indexed f
+(%@~) = iover
 {-# INLINE (%@~) #-}
 
+-- | Replace every target of an 'IndexedSetter', 'IndexedLens' or 'IndexedTraversal'
+-- with access to the index.
+--
+-- @
+-- ('.@~') ≡ 'iset'
+-- @
+--
+-- When you do not need access to the index then ('.~') is more liberal in what it can accept.
+--
+-- @
+-- l '.~' b ≡ l '.@~' 'const' b
+-- @
+--
+-- @
+-- ('.@~') :: 'IndexedSetter' i s t a b    -> (i -> b) -> s -> t
+-- ('.@~') :: 'IndexedLens' i s t a b      -> (i -> b) -> s -> t
+-- ('.@~') :: 'IndexedTraversal' i s t a b -> (i -> b) -> s -> t
+-- @
+(.@~) :: AnIndexedSetter i s t a b -> (i -> b) -> s -> t
+l .@~ f = runIdentity #. l (Identity #. Indexed (const . f))
+{-# INLINE (.@~) #-}
+
 -- | Adjust every target in the current state of an 'IndexedSetter', 'IndexedLens' or 'IndexedTraversal'
 -- with access to the index.
 --
@@ -1188,6 +1238,29 @@ l %@~ f = l %~ Indexed f
 l %@= f = State.modify (l %@~ f)
 {-# INLINE (%@=) #-}
 
+-- | This is an alias for ('%@=').
+imodifying :: MonadState s m => AnIndexedSetter i s s a b -> (i -> a -> b) -> m ()
+imodifying l f = State.modify (iover l f)
+{-# INLINE imodifying #-}
+
+-- | Replace every target in the current state of an 'IndexedSetter', 'IndexedLens' or 'IndexedTraversal'
+-- with access to the index.
+--
+-- When you do not need access to the index then ('.=') is more liberal in what it can accept.
+--
+-- @
+-- l '.=' b ≡ l '.@=' 'const' b
+-- @
+--
+-- @
+-- ('.@=') :: 'MonadState' s m => 'IndexedSetter' i s s a b    -> (i -> b) -> m ()
+-- ('.@=') :: 'MonadState' s m => 'IndexedLens' i s s a b      -> (i -> b) -> m ()
+-- ('.@=') :: 'MonadState' s m => 'IndexedTraversal' i s t a b -> (i -> b) -> m ()
+-- @
+(.@=) :: MonadState s m => AnIndexedSetter i s s a b -> (i -> b) -> m ()
+l .@= f = State.modify (l .@~ f)
+{-# INLINE (.@=) #-}
+
 ------------------------------------------------------------------------------
 -- Arrows
 ------------------------------------------------------------------------------
@@ -1228,7 +1301,7 @@ assignA l p = arr (flip $ set l) &&& p >>> arr (uncurry id)
 ------------------------------------------------------------------------------
 
 -- | 'mapOf' is a deprecated alias for 'over'.
-mapOf :: Profunctor p => Setting p s t a b -> p a b -> s -> t
+mapOf :: ASetter s t a b -> (a -> b) -> s -> t
 mapOf = over
 {-# INLINE mapOf #-}
 {-# DEPRECATED mapOf "Use `over`" #-}
