@@ -6,10 +6,7 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-
-#ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
-#endif
 
 #ifndef MIN_VERSION_containers
 #define MIN_VERSION_containers(x,y,z) 1
@@ -17,7 +14,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Traversal
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -116,11 +113,20 @@ module Control.Lens.Traversal
   , imapAccumROf
   , imapAccumLOf
 
+  -- * Reflection
+  , traverseBy
+  , traverseByOf
+  , sequenceBy
+  , sequenceByOf
+
   -- * Implementation Details
   , Bazaar(..), Bazaar'
   , Bazaar1(..), Bazaar1'
   , loci
   , iloci
+
+  -- * Fusion
+  , confusing
   ) where
 
 import Control.Applicative as Applicative
@@ -128,23 +134,33 @@ import Control.Applicative.Backwards
 import Control.Category
 import Control.Comonad
 import Control.Lens.Fold
-import Control.Lens.Getter (coerced)
+import Control.Lens.Getter (Getting, IndexedGetting, getting)
 import Control.Lens.Internal.Bazaar
 import Control.Lens.Internal.Context
 import Control.Lens.Internal.Indexed
 import Control.Lens.Lens
+import Control.Lens.Setter (ASetter, AnIndexedSetter, isets, sets)
 import Control.Lens.Type
 import Control.Monad
 import Control.Monad.Trans.State.Lazy
 import Data.Bitraversable
+#if __GLASGOW_HASKELL__ < 710
+import Data.Foldable (Foldable)
+#endif
 import Data.Functor.Compose
+import Data.Functor.Kan.Rift
+import Data.Functor.Yoneda
 import Data.Int
 import Data.IntMap as IntMap
 import Data.Map as Map
+import Data.Sequence (Seq, mapWithIndex)
+import Data.Vector as Vector (Vector, imap)
 import Data.Monoid
 import Data.Profunctor
 import Data.Profunctor.Rep
+import Data.Profunctor.Sieve
 import Data.Profunctor.Unsafe
+import Data.Reflection
 import Data.Semigroup.Traversable
 import Data.Tagged
 import Data.Traversable
@@ -160,6 +176,7 @@ import Prelude hiding ((.),id)
 -- >>> import Data.Maybe (fromMaybe)
 -- >>> import Debug.SimpleReflect.Vars
 -- >>> import Data.Void
+-- >>> import Data.List (sort)
 -- >>> import System.Timeout (timeout)
 -- >>> let timingOut :: NFData a => a -> IO a; timingOut = fmap (fromMaybe (error "timeout")) . timeout (5*10^6) . evaluate . force
 
@@ -257,7 +274,7 @@ type Traversing1' p f s a = Traversing1 p f s s a a
 -- 'traverseOf' :: 'Functor' f => 'Lens' s t a b      -> (a -> f b) -> s -> f t
 -- 'traverseOf' :: 'Applicative' f => 'Traversal' s t a b -> (a -> f b) -> s -> f t
 -- @
-traverseOf :: Over p f s t a b -> p a (f b) -> s -> f t
+traverseOf :: LensLike f s t a b -> (a -> f b) -> s -> f t
 traverseOf = id
 {-# INLINE traverseOf #-}
 
@@ -286,7 +303,7 @@ traverseOf = id
 -- 'forOf' :: 'Functor' f => 'Lens' s t a b -> s -> (a -> f b) -> f t
 -- 'forOf' :: 'Applicative' f => 'Traversal' s t a b -> s -> (a -> f b) -> f t
 -- @
-forOf :: Over p f s t a b -> s -> p a (f b) -> f t
+forOf :: LensLike f s t a b -> s -> (a -> f b) -> f t
 forOf = flip
 {-# INLINE forOf #-}
 
@@ -326,7 +343,7 @@ sequenceAOf l = l id
 -- 'mapMOf' :: 'Monad' m => 'Lens' s t a b      -> (a -> m b) -> s -> m t
 -- 'mapMOf' :: 'Monad' m => 'Traversal' s t a b -> (a -> m b) -> s -> m t
 -- @
-mapMOf :: Profunctor p => Over p (WrappedMonad m) s t a b -> p a (m b) -> s -> m t
+mapMOf :: LensLike (WrappedMonad m) s t a b -> (a -> m b) -> s -> m t
 mapMOf l cmd = unwrapMonad #. l (WrapMonad #. cmd)
 {-# INLINE mapMOf #-}
 
@@ -346,7 +363,7 @@ mapMOf l cmd = unwrapMonad #. l (WrapMonad #. cmd)
 -- 'forMOf' :: 'Monad' m => 'Lens' s t a b      -> s -> (a -> m b) -> m t
 -- 'forMOf' :: 'Monad' m => 'Traversal' s t a b -> s -> (a -> m b) -> m t
 -- @
-forMOf :: Profunctor p => Over p (WrappedMonad m) s t a b -> s -> p a (m b) -> m t
+forMOf :: LensLike (WrappedMonad m) s t a b -> s -> (a -> m b) -> m t
 forMOf l a cmd = unwrapMonad (l (WrapMonad #. cmd) a)
 {-# INLINE forMOf #-}
 
@@ -408,7 +425,7 @@ transposeOf l = getZipList #. l ZipList
 -- @
 -- 'mapAccumROf' :: 'LensLike' ('Backwards' ('State' acc)) s t a b -> (acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
 -- @
-mapAccumROf :: Conjoined p => Over p (Backwards (State acc)) s t a b -> p acc (a -> (acc, b)) -> acc -> s -> (acc, t)
+mapAccumROf :: LensLike (Backwards (State acc)) s t a b -> (acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
 mapAccumROf = mapAccumLOf . backwards
 {-# INLINE mapAccumROf #-}
 
@@ -431,9 +448,9 @@ mapAccumROf = mapAccumLOf . backwards
 -- 'mapAccumLOf' l f acc0 s = 'swap' ('runState' (l (\a -> 'state' (\acc -> 'swap' (f acc a))) s) acc0)
 -- @
 --
-mapAccumLOf :: Conjoined p => Over p (State acc) s t a b -> p acc (a -> (acc, b)) -> acc -> s -> (acc, t)
+mapAccumLOf :: LensLike (State acc) s t a b -> (acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
 mapAccumLOf l f acc0 s = swap (runState (l g s) acc0) where
-   g = cotabulate $ \wa -> state $ \acc -> swap (corep f (acc <$ wa) (extract wa))
+   g a = state $ \acc -> swap (f acc a)
 -- This would be much cleaner if the argument order for the function was swapped.
 {-# INLINE mapAccumLOf #-}
 
@@ -501,6 +518,9 @@ iloci f w = getCompose (runBazaar w (Compose #. Indexed (\i -> fmap (indexed sel
 -- >>> (a,b,c) & partsOf each .~ [x,y]
 -- (x,y,c)
 --
+-- >>> ('b', 'a', 'd', 'c') & partsOf each %~ sort
+-- ('a','b','c','d')
+--
 -- So technically, this is only a 'Lens' if you do not change the number of results it returns.
 --
 -- When applied to a 'Fold' the result is merely a 'Getter'.
@@ -601,7 +621,7 @@ holesOf l s = unTagged
       in f (ins b) (unsafeOuts b))
      (Tagged $ let
         f [] _ = []
-        f (wx:xs) g = Pretext (\wxfy -> g . (:Prelude.map extract xs) <$> corep wxfy wx) : f xs (g . (extract wx:))
+        f (wx:xs) g = Pretext (\wxfy -> g . (:Prelude.map extract xs) <$> cosieve wxfy wx) : f xs (g . (extract wx:))
       in f (pins b) (unsafeOuts b))
     :: Tagged (p a b) [Pretext p a a t]
   ) where b = l sell s
@@ -625,6 +645,9 @@ holesOf l s = unTagged
 -- >>> [1..10] ^. singular (ix 7)
 -- 8
 --
+-- >>> [] & singular traverse .~ 0
+-- []
+--
 -- @
 -- 'singular' :: 'Traversal' s t a a          -> 'Lens' s t a a
 -- 'singular' :: 'Fold' s a                   -> 'Getter' s a
@@ -639,8 +662,8 @@ singular l = conjoined
     (w:ws) -> unsafeOuts b . (:ws) <$> afb w
     []     -> unsafeOuts b . return <$> afb (error "singular: empty traversal"))
   (\pafb s -> let b = l sell s in case pins b of
-    (w:ws) -> unsafeOuts b . (:Prelude.map extract ws) <$> corep pafb w
-    []     -> unsafeOuts b . return                    <$> corep pafb (error "singular: empty traversal"))
+    (w:ws) -> unsafeOuts b . (:Prelude.map extract ws) <$> cosieve pafb w
+    []     -> unsafeOuts b . return                    <$> cosieve pafb (error "singular: empty traversal"))
 {-# INLINE singular #-}
 
 -- | This converts a 'Traversal' that you \"know\" will target only one element to a 'Lens'. It can also be
@@ -649,6 +672,9 @@ singular l = conjoined
 -- The resulting 'Lens' or 'Getter' will be partial if the 'Traversal' targets nothing
 -- or more than one element.
 --
+-- >>> [] & unsafeSingular traverse .~ 0
+-- *** Exception: unsafeSingular: empty traversal
+--
 -- @
 -- 'unsafeSingular' :: 'Traversal' s t a b          -> 'Lens' s t a b
 -- 'unsafeSingular' :: 'Fold' s a                   -> 'Getter' s a
@@ -664,7 +690,7 @@ unsafeSingular l = conjoined
     []  -> error "unsafeSingular: empty traversal"
     _   -> error "unsafeSingular: traversing multiple results")
   (\pafb s -> let b = inline l sell s in case pins b of
-    [w] -> unsafeOuts b . return <$> corep pafb w
+    [w] -> unsafeOuts b . return <$> cosieve pafb w
     []  -> error "unsafeSingular: empty traversal"
     _   -> error "unsafeSingular: traversing multiple results")
 {-# INLINE unsafeSingular #-}
@@ -674,7 +700,7 @@ unsafeSingular l = conjoined
 ------------------------------------------------------------------------------
 
 ins :: Bizarre (->) w => w a b t -> [a]
-ins = toListOf (coerced bazaar)
+ins = toListOf (getting bazaar)
 {-# INLINE ins #-}
 
 wins :: (Bizarre p w, Corepresentable p, Comonad (Corep p)) => w a b t -> [a]
@@ -763,7 +789,7 @@ beside :: (Representable q, Applicative (Rep q), Applicative f, Bitraversable r)
        => Optical p q f s t a b
        -> Optical p q f s' t' a b
        -> Optical p q f (r s s') (r t t') a b
-beside l r f = tabulate $ getCompose #. bitraverse (Compose #. rep (l f)) (Compose #. rep (r f))
+beside l r f = tabulate $ getCompose #. bitraverse (Compose #. sieve (l f)) (Compose #. sieve (r f))
 {-# INLINE beside #-}
 
 -- | Visit the first /n/ targets of a 'Traversal', 'Fold', 'Getter' or 'Lens'.
@@ -795,7 +821,7 @@ taking :: (Conjoined p, Applicative f)
        -> Over p f s t a a
 taking n l = conjoined
   (\ afb s  -> let b = inline l sell s in outs b <$> traverse afb          (take n $ ins b))
-  (\ pafb s -> let b = inline l sell s in outs b <$> traverse (corep pafb) (take n $ pins b))
+  (\ pafb s -> let b = inline l sell s in outs b <$> traverse (cosieve pafb) (take n $ pins b))
 {-# INLINE taking #-}
 
 -- | Visit all but the first /n/ targets of a 'Traversal', 'Fold', 'Getter' or 'Lens'.
@@ -822,7 +848,7 @@ taking n l = conjoined
 -- @
 dropping :: (Conjoined p, Applicative f) => Int -> Over p (Indexing f) s t a a -> Over p f s t a a
 dropping n l pafb s = snd $ runIndexing (l paifb s) 0 where
-  paifb = cotabulate $ \wa -> Indexing $ \i -> let i' = i + 1 in i' `seq` (i', if i < n then pure (extract wa) else corep pafb wa)
+  paifb = cotabulate $ \wa -> Indexing $ \i -> let i' = i + 1 in i' `seq` (i', if i < n then pure (extract wa) else cosieve pafb wa)
 {-# INLINE dropping #-}
 
 ------------------------------------------------------------------------------
@@ -839,11 +865,11 @@ dropping n l pafb s = snd $ runIndexing (l paifb s) 0 where
 -- as such, use 'Control.Lens.Lens.cloneLens'.
 --
 -- Note: It is usually better to use 'Control.Lens.Reified.ReifiedTraversal' and
--- 'Control.Lens.Reified.reflectTraversal' than to 'cloneTraversal'. The
+-- 'Control.Lens.Reified.runTraversal' than to 'cloneTraversal'. The
 -- former can execute at full speed, while the latter needs to round trip through
 -- the 'Bazaar'.
 --
--- >>> let foo l a = (view (coerced (cloneTraversal l)) a, set (cloneTraversal l) 10 a)
+-- >>> let foo l a = (view (getting (cloneTraversal l)) a, set (cloneTraversal l) 10 a)
 -- >>> foo both ("hello","world")
 -- ("helloworld",(10,10))
 --
@@ -857,7 +883,7 @@ cloneTraversal l f = bazaar f . l sell
 -- | Clone a 'Traversal' yielding an 'IndexPreservingTraversal' that passes through
 -- whatever index it is composed with.
 cloneIndexPreservingTraversal :: ATraversal s t a b -> IndexPreservingTraversal s t a b
-cloneIndexPreservingTraversal l pafb = cotabulate $ \ws -> runBazaar (l sell (extract ws)) $ \a -> corep pafb (a <$ ws)
+cloneIndexPreservingTraversal l pafb = cotabulate $ \ws -> runBazaar (l sell (extract ws)) $ \a -> cosieve pafb (a <$ ws)
 {-# INLINE cloneIndexPreservingTraversal #-}
 
 -- | Clone an 'IndexedTraversal' yielding an 'IndexedTraversal' with the same index.
@@ -873,7 +899,7 @@ cloneTraversal1 l f = bazaar1 f . l sell
 -- | Clone a 'Traversal1' yielding an 'IndexPreservingTraversal1' that passes through
 -- whatever index it is composed with.
 cloneIndexPreservingTraversal1 :: ATraversal1 s t a b -> IndexPreservingTraversal1 s t a b
-cloneIndexPreservingTraversal1 l pafb = cotabulate $ \ws -> runBazaar1 (l sell (extract ws)) $ \a -> corep pafb (a <$ ws)
+cloneIndexPreservingTraversal1 l pafb = cotabulate $ \ws -> runBazaar1 (l sell (extract ws)) $ \a -> cosieve pafb (a <$ ws)
 {-# INLINE cloneIndexPreservingTraversal1 #-}
 
 -- | Clone an 'IndexedTraversal1' yielding an 'IndexedTraversal1' with the same index.
@@ -935,8 +961,8 @@ iforOf = flip . itraverseOf
 -- 'imapMOf' :: 'Monad' m => 'IndexedTraversal'  i s t a b -> (i -> a -> m b) -> s -> m t
 -- 'imapMOf' :: 'Bind'  m => 'IndexedTraversal1' i s t a b -> (i -> a -> m b) -> s -> m t
 -- @
-imapMOf :: (Indexed i a (WrappedMonad m b) -> s -> WrappedMonad m t) -> (i -> a -> m b) -> s -> m t
-imapMOf l = mapMOf l .# Indexed
+imapMOf :: Over (Indexed i) (WrappedMonad m) s t a b  -> (i -> a -> m b) -> s -> m t
+imapMOf l cmd = unwrapMonad #. l (WrapMonad #. Indexed cmd)
 {-# INLINE imapMOf #-}
 
 -- | Map each element of a structure targeted by a 'Lens' to a monadic action,
@@ -969,7 +995,7 @@ iforMOf = flip . imapMOf
 -- 'imapAccumROf' :: 'IndexedTraversal' i s t a b -> (i -> acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
 -- @
 imapAccumROf :: Over (Indexed i) (Backwards (State acc)) s t a b -> (i -> acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
-imapAccumROf l = mapAccumROf l .# Indexed
+imapAccumROf = imapAccumLOf . backwards
 {-# INLINE imapAccumROf #-}
 
 -- | Generalizes 'Data.Traversable.mapAccumL' to an arbitrary 'IndexedTraversal' with access to the index.
@@ -985,7 +1011,8 @@ imapAccumROf l = mapAccumROf l .# Indexed
 -- 'imapAccumLOf' :: 'IndexedTraversal' i s t a b -> (i -> acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
 -- @
 imapAccumLOf :: Over (Indexed i) (State acc) s t a b -> (i -> acc -> a -> (acc, b)) -> acc -> s -> (acc, t)
-imapAccumLOf l = mapAccumLOf l .# Indexed
+imapAccumLOf l f acc0 s = swap (runState (l (Indexed g) s) acc0) where
+  g i a = state $ \acc -> swap (f i acc a)
 {-# INLINE imapAccumLOf #-}
 
 ------------------------------------------------------------------------------
@@ -995,7 +1022,23 @@ imapAccumLOf l = mapAccumLOf l .# Indexed
 -- | Traverse any 'Traversable' container. This is an 'IndexedTraversal' that is indexed by ordinal position.
 traversed :: Traversable f => IndexedTraversal Int (f a) (f b) a b
 traversed = conjoined traverse (indexing traverse)
-{-# INLINE traversed #-}
+{-# INLINE [0] traversed #-}
+
+imapList :: (Int -> a -> b) -> [a] -> [b]
+imapList f = go 0
+  where
+    go i (x:xs) = f i x : go (i+1) xs
+    go _ []     = []
+{-# INLINE imapList #-}
+
+{-# RULES
+"traversed -> mapped"     traversed = sets fmap          :: Functor f => ASetter (f a) (f b) a b;
+"traversed -> folded"     traversed = folded             :: Foldable f => Getting (Endo r) (f a) a;
+"traversed -> ifolded"    traversed = folded             :: Foldable f => IndexedGetting Int (Endo r) (f a) a;
+"traversed -> imapList"   traversed = isets imapList     :: AnIndexedSetter Int [a] [b] a b;
+"traversed -> imapSeq"    traversed = isets mapWithIndex :: AnIndexedSetter Int (Seq a) (Seq b) a b;
+"traversed -> imapVector" traversed = isets Vector.imap  :: AnIndexedSetter Int (Vector a) (Vector b) a b;
+ #-}
 
 -- | Traverse any 'Traversable1' container. This is an 'IndexedTraversal1' that is indexed by ordinal position.
 traversed1 :: Traversable1 f => IndexedTraversal1 Int (f a) (f b) a b
@@ -1136,8 +1179,8 @@ elements = elementsOf traverse
 -- @
 -- 'failover' :: Alternative m => Traversal s t a b -> (a -> b) -> s -> m t
 -- @
-failover :: (Profunctor p, Alternative m) => Over p ((,) Any) s t a b -> p a b -> s -> m t
-failover l pafb s = case l ((,) (Any True) `rmap` pafb) s of
+failover :: Alternative m => LensLike ((,) Any) s t a b -> (a -> b) -> s -> m t
+failover l afb s = case l ((,) (Any True) . afb) s of
   (Any True, t)  -> pure t
   (Any False, _) -> Applicative.empty
 {-# INLINE failover #-}
@@ -1148,7 +1191,9 @@ failover l pafb s = case l ((,) (Any True) `rmap` pafb) s of
 -- 'ifailover' :: Alternative m => IndexedTraversal i s t a b -> (i -> a -> b) -> s -> m t
 -- @
 ifailover :: Alternative m => Over (Indexed i) ((,) Any) s t a b -> (i -> a -> b) -> s -> m t
-ifailover l f = failover l (Indexed f)
+ifailover l iafb s = case l ((,) (Any True) `rmap` Indexed iafb) s of
+  (Any True, t) -> pure t
+  (Any False, _) -> Applicative.empty
 {-# INLINE ifailover #-}
 
 -- | Try the first 'Traversal' (or 'Fold'), falling back on the second 'Traversal' (or 'Fold') if it returns no entries.
@@ -1194,11 +1239,12 @@ ifailover l f = failover l (Indexed f)
 -- 'failing' :: 'IndexedLens' i s t a b      -> 'IndexedTraversal' i s t a b -> 'IndexedTraversal' i s t a b
 -- 'failing' :: 'IndexedGetter' i s a        -> 'IndexedGetter' i s a        -> 'IndexedFold' i s a
 -- @
-failing :: (Conjoined p, Applicative f) => Traversing p f s t a b -> Traversing p f s t a b -> Over p f s t a b
+failing :: (Conjoined p, Applicative f) => Traversing p f s t a b -> Over p f s t a b -> Over p f s t a b
 failing l r pafb s = case pins b of
-  [] -> runBazaarT (r sell s) pafb
-  xs -> unsafeOuts b <$> traverse (corep pafb) xs
+  [] -> r pafb s
+  _  -> bazaar pafb b
   where b = l sell s
+
 infixl 5 `failing`
 
 -- | Try the second traversal. If it returns no entries, try again with all entries from the first traversal, recursively.
@@ -1214,5 +1260,63 @@ deepOf :: (Conjoined p, Applicative f) => LensLike f s t s t -> Traversing p f s
 deepOf r l pafb = go
   where go s = case pins b of
           [] -> r go s
-          xs -> unsafeOuts b <$> traverse (corep pafb) xs
+          xs -> unsafeOuts b <$> traverse (cosieve pafb) xs
           where b = l sell s
+
+-- | "Fuse" a 'Traversal' by reassociating all of the '\<*\>' operations to the
+-- left and fusing all of the 'fmap' calls into one. This is particularly
+-- useful when constructing a 'Traversal' using operations from GHC.Generics.
+--
+-- Given a pair of 'Traversal's 'foo' and 'bar',
+--
+-- @
+-- 'confusing' (foo.bar) = foo.bar
+-- @
+--
+-- However, @foo@ and @bar@ are each going to use the 'Applicative' they are given.
+--
+-- 'confusing' exploits the 'Yoneda' lemma to merge their separate uses of 'fmap' into a single 'fmap'.
+-- and it further exploits an interesting property of the right Kan lift (or 'Rift') to left associate
+-- all of the uses of '(<*>)' to make it possible to fuse together more fmaps.
+--
+-- This is particularly effective when the choice of functor 'f' is unknown at compile
+-- time or when the 'Traversal' @foo.bar@ in the above description is recursive or complex
+-- enough to prevent inlining.
+--
+-- 'Control.Lens.Lens.fusing' is a version of this combinator suitable for fusing lenses.
+--
+-- @
+-- 'confusing' :: 'Traversal' s t a b -> 'Traversal' s t a b
+-- @
+confusing :: Applicative f => LensLike (Rift (Yoneda f) (Yoneda f)) s t a b -> LensLike f s t a b
+confusing t = \f -> lowerYoneda . lowerRift . t (liftRiftYoneda . f)
+  where
+  liftRiftYoneda :: Applicative f => f a -> Rift (Yoneda f) (Yoneda f) a
+  liftRiftYoneda fa = Rift (`yap` fa)
+  {-# INLINE liftRiftYoneda #-}
+
+  yap :: Applicative f => Yoneda f (a -> b) -> f a -> Yoneda f b
+  yap (Yoneda k) fa = Yoneda (\ab_r -> k (ab_r .) <*> fa)
+  {-# INLINE yap #-}
+
+{-# INLINE confusing #-}
+
+-- | Traverse a container using a specified 'Applicative'.
+--
+-- This is like 'traverseBy' where the 'Traversable' instance can be specified by any 'Traversal'
+--
+-- @
+-- 'traverseByOf' 'traverse' ≡ 'traverseBy'
+-- @
+traverseByOf :: Traversal s t a b -> (forall x. x -> f x) -> (forall x y. f (x -> y) -> f x -> f y) -> (a -> f b) -> s -> f t
+traverseByOf l pur app f = reifyApplicative pur app (l (ReflectedApplicative #. f))
+
+-- | Sequence a container using a specified 'Applicative'.
+--
+-- This is like 'traverseBy' where the 'Traversable' instance can be specified by any 'Traversal'
+--
+-- @
+-- 'sequenceByOf' 'traverse' ≡ 'sequenceBy'
+-- @
+sequenceByOf :: Traversal s t (f b) b -> (forall x. x -> f x) -> (forall x y. f (x -> y) -> f x -> f y) -> s -> f t
+sequenceByOf l pur app = reifyApplicative pur app (l ReflectedApplicative)
