@@ -6,17 +6,30 @@
 {-# LANGUAGE TypeSynonymInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE UndecidableInstances #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 706
+
+#if __GLASGOW_HASKELL__ >= 706
 {-# LANGUAGE PolyKinds #-}
 #endif
+
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
+
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE PatternSynonyms #-}
+{-# LANGUAGE ViewPatterns #-}
+#endif
+
 {-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
+
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Wrapped
--- Copyright   :  (C) 2012-14 Edward Kmett, Michael Sloan
+-- Copyright   :  (C) 2012-15 Edward Kmett, Michael Sloan
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -57,8 +70,14 @@ module Control.Lens.Wrapped
   -- * Operations
   , op
   , ala, alaf
+#if __GLASGOW_HASKELL__ >= 710
+  -- * Pattern Synonyms
+  , pattern Wrapped
+  , pattern Unwrapped
+#endif
   ) where
 
+
 import           Control.Applicative
 import           Control.Arrow
 import           Control.Applicative.Backwards
@@ -66,6 +85,9 @@ import           Control.Comonad.Trans.Traced
 import           Control.Exception
 import           Control.Lens.Getter
 import           Control.Lens.Iso
+#if __GLASGOW_HASKELL__ >= 710
+import           Control.Lens.Review
+#endif
 import           Control.Monad.Trans.Cont
 import           Control.Monad.Trans.Error
 import           Control.Monad.Trans.Identity
@@ -91,6 +113,7 @@ import           Data.IntSet as IntSet
 import           Data.IntMap as IntMap
 import           Data.HashSet as HashSet
 import           Data.HashMap.Lazy as HashMap
+import           Data.List.NonEmpty
 import           Data.Map as Map
 import           Data.Monoid
 import qualified Data.Semigroup as S
@@ -102,6 +125,16 @@ import           Data.Vector.Primitive as Prim
 import           Data.Vector.Unboxed as Unboxed
 import           Data.Vector.Storable as Storable
 
+#if MIN_VERSION_base(4,6,0)
+import           Data.Ord (Down(Down))
+#else
+import           GHC.Exts (Down(Down))
+#endif
+
+#ifdef HLINT
+{-# ANN module "HLint: ignore Use uncurry" #-}
+#endif
+
 -- $setup
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Control.Lens
@@ -113,6 +146,16 @@ class Wrapped s where
   -- | An isomorphism between @s@ and @a@.
   _Wrapped' :: Iso' s (Unwrapped s)
 
+#if __GLASGOW_HASKELL__ >= 710
+
+pattern Wrapped a <- (view _Wrapped -> a) where
+  Wrapped a = review _Wrapped a
+
+pattern Unwrapped a <- (view _Unwrapped -> a) where
+  Unwrapped a = review _Unwrapped a
+
+#endif
+
 -- This can be used to help inference between the wrappers
 class Wrapped s => Rewrapped (s :: *) (t :: *)
 
@@ -121,6 +164,7 @@ instance (Rewrapped s t, Rewrapped t s) => Rewrapping s t
 
 _Unwrapped' :: Wrapped s => Iso' (Unwrapped s) s
 _Unwrapped' = from _Wrapped'
+{-# INLINE _Unwrapped' #-}
 
 -- | Work under a newtype wrapper.
 --
@@ -189,6 +233,12 @@ instance Wrapped (ZipList a) where
   _Wrapped' = iso getZipList ZipList
   {-# INLINE _Wrapped' #-}
 
+instance (t ~ NonEmpty b) => Rewrapped (NonEmpty a) t
+instance Wrapped (NonEmpty a) where
+  type Unwrapped (NonEmpty a) = (a, [a])
+  _Wrapped' = iso (\(a :| as) -> (a, as)) (\(a,as) -> a :| as)
+  {-# INLINE _Wrapped' #-}
+
 instance (t ~ Const a' x') => Rewrapped (Const a x) t
 instance Wrapped (Const a x) where
   type Unwrapped (Const a x) = a
@@ -213,18 +263,31 @@ instance Wrapped (First a) where
   _Wrapped' = iso getFirst First
   {-# INLINE _Wrapped' #-}
 
-instance (t ~ Last b) => Rewrapped (Last b) t
+instance (t ~ Last b) => Rewrapped (Last a) t
 instance Wrapped (Last a) where
   type Unwrapped (Last a) = Maybe a
   _Wrapped' = iso getLast Last
   {-# INLINE _Wrapped' #-}
 
-instance (t ~ ArrowMonad m' a', ArrowApply m, ArrowApply m') => Rewrapped (ArrowMonad m a) t
-instance ArrowApply m => Wrapped (ArrowMonad m a) where
+#if MIN_VERSION_base(4,8,0)
+instance (t ~ Alt g b) => Rewrapped (Alt f a) t
+instance Wrapped (Alt f a) where
+  type Unwrapped (Alt f a) = f a
+  _Wrapped' = iso getAlt Alt
+  {-# INLINE _Wrapped' #-}
+#endif
+
+instance (t ~ ArrowMonad m' a', ArrowApply m) => Rewrapped (ArrowMonad m a) t
+instance Wrapped (ArrowMonad m a) where
   type Unwrapped (ArrowMonad m a) = m () a
   _Wrapped' = iso getArrowMonad ArrowMonad
   {-# INLINE _Wrapped' #-}
 
+instance t ~ Down a => Rewrapped (Down a) t
+instance Wrapped (Down a) where
+  type Unwrapped (Down a) = a
+  _Wrapped' = iso (\(Down a) -> a) Down
+
 -- * transformers
 
 instance (t ~ Backwards g b) => Rewrapped (Backwards f a) t
@@ -579,7 +642,7 @@ failedAssertion :: AssertionFailed -> String
 failedAssertion (AssertionFailed x) = x
 {-# INLINE failedAssertion #-}
 
-getArrowMonad :: ArrowApply m  => ArrowMonad m a -> m () a
+getArrowMonad :: ArrowMonad m a -> m () a
 getArrowMonad (ArrowMonad x) = x
 {-# INLINE getArrowMonad #-}
 
@@ -655,7 +718,15 @@ _Unwrapping _ = from _Wrapped
 --
 -- >>> ala Product foldMap [1,2,3,4]
 -- 24
-ala :: Rewrapping s t => (Unwrapped s -> s) -> ((Unwrapped t -> t) -> e -> s) -> e -> Unwrapped s
+--
+--
+-- You may want to think of this combinator as having the following, simpler, type.
+--
+-- @
+-- ala :: Rewrapping s t => (Unwrapped s -> s) -> ((Unwrapped t -> t) -> e -> s) -> e -> Unwrapped s
+-- @
+
+ala :: (Functor f, Rewrapping s t) => (Unwrapped s -> s) -> ((Unwrapped t -> t) -> f s) -> f (Unwrapped s)
 ala = au . _Wrapping
 {-# INLINE ala #-}
 
@@ -663,8 +734,12 @@ ala = au . _Wrapping
 --
 -- As with '_Wrapping', the user supplied function for the newtype is /ignored/.
 --
+-- @
+-- alaf :: Rewrapping s t => (Unwrapped s -> s) -> ((r ->  t) -> e -> s) -> (r -> Unwrapped t) -> e -> Unwrapped s
+-- @
+--
 -- >>> alaf Sum foldMap Prelude.length ["hello","world"]
 -- 10
-alaf :: (Profunctor p, Rewrapping s t) => (Unwrapped s -> s) -> (p r t -> e -> s) -> p r (Unwrapped t) -> e -> Unwrapped s
+alaf :: (Functor f, Functor g, Rewrapping s t) => (Unwrapped s -> s) -> (f t -> g s) -> f (Unwrapped t) -> g (Unwrapped s)
 alaf = auf . _Unwrapping
 {-# INLINE alaf #-}
