@@ -1,9 +1,15 @@
-{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE FlexibleContexts #-}
+
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Bits.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -22,9 +28,12 @@ module Data.Bits.Lens
 import Control.Lens
 import Control.Monad.State
 import Data.Bits
-import Data.Functor
 import Data.Word
 
+#if !MIN_VERSION_base(4,8,0)
+import Data.Functor
+#endif
+
 -- $setup
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Data.Word
