@@ -1,8 +1,13 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE PatternSynonyms #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.ByteString.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -13,6 +18,10 @@ module Data.ByteString.Lens
   ( IsByteString(..)
   , unpackedBytes
   , unpackedChars
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern Bytes
+  , pattern Chars
+#endif
   ) where
 
 import           Control.Lens
@@ -101,6 +110,14 @@ unpackedBytes :: IsByteString t => Iso' t [Word8]
 unpackedBytes = from packedBytes
 {-# INLINE unpackedBytes #-}
 
+#if __GLASGOW_HASKELL__ >= 710
+pattern Bytes b <- (view unpackedBytes -> b) where
+  Bytes b = review unpackedBytes b
+  
+pattern Chars b <- (view unpackedChars -> b) where
+  Chars b = review unpackedChars b
+#endif
+
 -- | 'Data.ByteString.Char8.unpack' (or 'Data.ByteString.Char8.pack') a list of characters into a strict (or lazy) 'ByteString'
 --
 -- When writing back to the 'ByteString' it is assumed that every 'Char'
