@@ -1,9 +1,14 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE PatternSynonyms #-}
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.ByteString.Strict.Lens
--- Copyright   :  (C) 2012-2014 Edward Kmett
+-- Copyright   :  (C) 2012-2015 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -13,6 +18,10 @@
 module Data.ByteString.Strict.Lens
   ( packedBytes, unpackedBytes, bytes
   , packedChars, unpackedChars, chars
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern Bytes
+  , pattern Chars
+#endif
   ) where
 
 import Control.Lens
@@ -70,7 +79,7 @@ unpackedBytes = from packedBytes
 -- Note that when just using this as a 'Setter', @'setting' 'Data.ByteString.map'@
 -- can be more efficient.
 bytes :: IndexedTraversal' Int ByteString Word8
-bytes = traversedStrictTree 0
+bytes = traversedStrictTree
 {-# INLINE bytes #-}
 
 -- | 'Data.ByteString.Char8.pack' (or 'Data.ByteString.Char8.unpack') a list of characters into a 'ByteString'
@@ -124,5 +133,14 @@ unpackedChars = from packedChars
 -- >>> anyOf chars (== 'h') "hello"
 -- True
 chars :: IndexedTraversal' Int ByteString Char
-chars = traversedStrictTree8 0
+chars = traversedStrictTree8
 {-# INLINE chars #-}
+
+#if __GLASGOW_HASKELL__ >= 710
+pattern Bytes b <- (view unpackedBytes -> b) where
+  Bytes b = review unpackedBytes b
+
+pattern Chars b <- (view unpackedChars -> b) where
+  Chars b = review unpackedChars b
+#endif
+
