@@ -1,12 +1,18 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE PatternSynonyms #-}
+#endif
+
 #ifndef MIN_VERSION_base
 #define MIN_VERSION_base(x,y,z) 1
 #endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Complex.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -22,12 +28,22 @@ module Data.Complex.Lens
   , _magnitude
   , _phase
   , _conjugate
+#if __GLASGOW_HASKELL__ >= 710
+  -- * Pattern Synonyms
+  , pattern Polar
+  , pattern Real
+  , pattern Imaginary
+  , pattern Conjugate
+#endif
   ) where
 
-import Control.Applicative
 import Control.Lens
 import Data.Complex
 
+#if !MIN_VERSION_base(4,8,0)
+import Control.Applicative
+#endif
+
 -- $setup
 -- >>> import Debug.SimpleReflect
 -- >>> let { a ≈ b = abs (a - b) < 1e-6; infix 4 ≈ }
@@ -41,11 +57,7 @@ import Data.Complex
 -- a * 2 :+ b
 --
 -- @'_realPart' :: 'Functor' f => (a -> f a) -> 'Complex' a -> f ('Complex' a)@
-#if MIN_VERSION_base(4,4,0)
 _realPart :: Lens' (Complex a) a
-#else
-_realPart :: RealFloat a => Lens' (Complex a) a
-#endif
 _realPart f (a :+ b) = (:+ b) <$> f a
 {-# INLINE _realPart #-}
 
@@ -58,11 +70,7 @@ _realPart f (a :+ b) = (:+ b) <$> f a
 -- a :+ b * 2
 --
 -- @'_imagPart' :: 'Functor' f => (a -> f a) -> 'Complex' a -> f ('Complex' a)@
-#if MIN_VERSION_base(4,4,0)
 _imagPart :: Lens' (Complex a) a
-#else
-_imagPart :: RealFloat a => Lens' (Complex a) a
-#endif
 _imagPart f (a :+ b) = (a :+) <$> f b
 {-# INLINE _imagPart #-}
 
@@ -80,6 +88,14 @@ _polar :: RealFloat a => Iso' (Complex a) (a,a)
 _polar = iso polar (uncurry mkPolar)
 {-# INLINE _polar #-}
 
+#if __GLASGOW_HASKELL__ >= 710
+pattern Polar m theta <- (view _polar -> (m, theta)) where
+  Polar m theta = review _polar (m, theta)
+
+pattern Real r      = r :+ 0
+pattern Imaginary i = 0 :+ i
+#endif
+
 -- | Access the 'magnitude' of a 'Complex' number.
 --
 -- >>> (10.0 :+ 20.0) & _magnitude *~ 2
@@ -131,3 +147,8 @@ _phase f c = setPhase <$> f theta
 _conjugate :: RealFloat a => Iso' (Complex a) (Complex a)
 _conjugate = involuted conjugate
 {-# INLINE _conjugate #-}
+
+#if __GLASGOW_HASKELL__ >= 710
+pattern Conjugate a <- (conjugate -> a) where
+  Conjugate a = conjugate a
+#endif
