@@ -1,9 +1,15 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE PatternSynonyms #-}
+{-# LANGUAGE ViewPatterns #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Dynamic.Lens
--- Copyright   :  (C) 2012-2014 Edward Kmett
+-- Copyright   :  (C) 2012-2015 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -12,6 +18,9 @@
 ----------------------------------------------------------------------------
 module Data.Dynamic.Lens
   ( AsDynamic(..)
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern Dynamic
+#endif
   ) where
 
 import Control.Exception
@@ -40,3 +49,8 @@ instance AsDynamic Dynamic where
 instance AsDynamic SomeException where
   _Dynamic = exception.prism' toDyn fromDynamic
   {-# INLINE _Dynamic #-}
+
+#if __GLASGOW_HASKELL__ >= 710
+pattern Dynamic a <- (preview _Dynamic -> Just a) where
+  Dynamic a = review _Dynamic a
+#endif
