@@ -1,8 +1,13 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
+
+#ifndef MIN_VERSION_containers
+#define MIN_VERSION_containers(x,y,z) 1
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Set.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -32,7 +37,11 @@ import Data.Set as Set
 --
 -- >>> over setmapped (+1) (fromList [1,2,3,4])
 -- fromList [2,3,4,5]
+#if MIN_VERSION_containers(0,5,2)
+setmapped :: Ord j => IndexPreservingSetter (Set i) (Set j) i j
+#else
 setmapped :: (Ord i, Ord j) => IndexPreservingSetter (Set i) (Set j) i j
+#endif
 setmapped = setting Set.map
 {-# INLINE setmapped #-}
 
