@@ -2,10 +2,15 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE TypeSynonymInstances #-}
+
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE PatternSynonyms #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Text.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -16,15 +21,27 @@ module Data.Text.Lens
   ( IsText(..)
   , unpacked
   , _Text
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern Text
+#endif
   ) where
 
-import           Control.Lens
+import           Control.Lens.Type
+#if __GLASGOW_HASKELL__ >= 710
+import           Control.Lens.Getter
+import           Control.Lens.Review
+#endif
+import           Control.Lens.Iso
+import           Control.Lens.Traversal
 import           Data.Text as Strict
 import qualified Data.Text.Strict.Lens as Strict
 import           Data.Text.Lazy as Lazy
 import qualified Data.Text.Lazy.Lens as Lazy
 import           Data.Text.Lazy.Builder
 
+-- $setup
+-- >>> import Control.Lens
+
 -- | Traversals for strict or lazy 'Text'
 class IsText t where
   -- | This isomorphism can be used to 'pack' (or 'unpack') strict or lazy 'Text'.
@@ -55,7 +72,7 @@ class IsText t where
 instance IsText String where
   packed = id
   {-# INLINE packed #-}
-  text = indexing traverse
+  text = traversed
   {-# INLINE text #-}
   builder = Lazy.packed . builder
   {-# INLINE builder #-}
@@ -89,6 +106,11 @@ _Text :: IsText t => Iso' t String
 _Text = from packed
 {-# INLINE _Text #-}
 
+#if __GLASGOW_HASKELL__ >= 710
+pattern Text a <- (view _Text -> a) where
+  Text a = review _Text a
+#endif
+
 instance IsText Strict.Text where
   packed = Strict.packed
   {-# INLINE packed #-}
@@ -104,3 +126,4 @@ instance IsText Lazy.Text where
   {-# INLINE builder #-}
   text = Lazy.text
   {-# INLINE text #-}
+
