@@ -1,9 +1,14 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
+
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Typeable.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -15,11 +20,14 @@ module Data.Typeable.Lens
   , _gcast
   ) where
 
-import Control.Applicative
 import Control.Lens
 import Data.Typeable
 import Data.Maybe
 
+#if !MIN_VERSION_base(4,8,0)
+import Control.Applicative
+#endif
+
 -- | A 'Traversal'' for working with a 'cast' of a 'Typeable' value.
 _cast :: (Typeable s, Typeable a) => Traversal' s a
 _cast f s = case cast s of
