@@ -1,8 +1,13 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE PatternSynonyms #-}
+{-# LANGUAGE ViewPatterns #-}
+#endif
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Numeric.Lens
--- Copyright   :  (C) 2012-14 Edward Kmett
+-- Copyright   :  (C) 2012-15 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -23,6 +28,9 @@ module Numeric.Lens
   , dividing
   , exponentiating
   , negated
+#if __GLASGOW_HASKELL__ >= 710
+  , pattern Integral
+#endif
   ) where
 
 import Control.Lens
@@ -45,6 +53,11 @@ integral = prism toInteger $ \ i -> let a = fromInteger i in
   then Right a
   else Left i
 
+#if __GLASGOW_HASKELL__ >= 710
+pattern Integral a <- (preview integral -> Just a) where
+  Integral a = review integral a
+#endif
+
 -- | A prism that shows and reads integers in base-2 through base-36
 --
 -- Note: This is an improper prism, since leading 0s are stripped when reading.
